# -*- coding: utf-8 -*-
"""Balafon : your django CRM"""

from __future__ import unicode_literals

import floppyforms.__future__ as forms


class HidableModelMultipleChoiceField(forms.ModelMultipleChoiceField):
    hidden_widget = forms.HiddenInput



# -*- coding: utf-8 -*-
"""common utils"""

from __future__ import unicode_literals

from bs4 import BeautifulSoup
from datetime import datetime
import logging
from six.moves.urllib.parse import urlparse
from importlib import import_module

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.urlresolvers import resolve, Resolver404
from django.http import HttpResponseRedirect, Http404
from django.utils.translation import ugettext as _

from rest_framework.renderers import JSONRenderer

from balafon.settings import get_allowed_homepages


logger = logging.getLogger("balafon_crm")


def log_error(view_func):
    """log error decorator"""
    def wrapper(request, *args, **kwargs):
        """wrapper"""
        try:
            return view_func(request, *args, **kwargs)
        except Http404:
            raise
        except:
            logger.exception("exception")
            raise
    return wrapper


def now_rounded():
    """datetime rounded to the minute"""
    now = datetime.now()
    return datetime(now.year, now.month, now.day, now.hour, now.minute, 0, 0)


class HttpResponseRedirectMailtoAllowed(HttpResponseRedirect):
    """Mailto redirect"""
    allowed_schemes = ['http', 'https', 'ftp', 'mailto']


def get_form_errors(response):
    """get form errors"""
    soup = BeautifulSoup(response.content, "html.parser")
    errors = soup.select('.field-error .label')
    return errors


def is_allowed_homepage(url_string):
    """return True is the current page can be set as homepage"""
    url = urlparse(url_string)
    try:
        safe_url = url.path
        resolved = resolve(safe_url)
    except Resolver404:
        return False
    if resolved.url_name in get_allowed_homepages():
        return True
    return False


def validate_rgb(value):
    """"check rgb"""
    wrong = False
    if not (len(value) == 7 or len(value) == 4):
        wrong = True
    else:
        if value[0] != "#":
            wrong = True
        else:
            try:
                int(value[1:], 16)
            except ValueError:
                wrong = True
    if wrong:
        raise ValidationError(_('RGB format (e.g. #123456) expected'))
    return False


class Utf8JSONRenderer(JSONRenderer):
    """Utf-8 support"""
    def render(self, *args, **kwargs):
        content = super(Utf8JSONRenderer, self).render(*args, **kwargs)
        return content.decode('utf-8')


def load_from_module(settings_key, default_value):
    """returns the form to be used for creating a new article"""
    full_class_name = getattr(settings, settings_key, '') or default_value
    if full_class_name:
        try:
            module_name, obj_name = full_class_name.rsplit('.', 1)
        except ValueError:
            raise ImportError("Unable to import {0}: full path is required".format(full_class_name))
        module = import_module(module_name)
        obj = getattr(module, obj_name)
        return obj
    return None



# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

from datetime import date, timedelta
from decimal import Decimal, InvalidOperation

from django.contrib import messages
from django.utils.translation import ugettext, ugettext_lazy as _

import floppyforms.__future__ as forms

from coop_cms.utils import RequestManager, RequestNotFound

from balafon.Crm.forms.base import BetterBsForm
from balafon.Crm.models import ActionStatus
from balafon.Store.models import (
    StoreManagementActionType, PricePolicy, StoreItemCategory, SaleAnalysisCode, VatRate, update_action_amount
)


class StoreManagementActionTypeAdminForm(forms.ModelForm):
    """admin form"""
    initial_show_amount_as_pre_tax = None

    def __init__(self, *args, **kwargs):
        super(StoreManagementActionTypeAdminForm, self).__init__(*args, **kwargs)
        if self.instance and self.instance.id and self.instance.action_type:
            self.initial_show_amount_as_pre_tax = self.instance.show_amount_as_pre_tax
            self.fields['readonly_status'].queryset = self.instance.action_type.allowed_status
        else:
            self.fields['readonly_status'].queryset = ActionStatus.objects.none()
            self.fields['readonly_status'].widget = forms.HiddenInput()

    def save(self, *args, **kwargs):
        show_amount_as_pre_tax = self.cleaned_data['show_amount_as_pre_tax']
        recalculate = False
        if self.instance and self.initial_show_amount_as_pre_tax != show_amount_as_pre_tax:
            recalculate = True
        ret = super(StoreManagementActionTypeAdminForm, self).save(*args, **kwargs)
        if recalculate:
            action_type = self.instance.action_type
            for action in action_type.action_set.all():
                try:
                    update_action_amount(action.sale)
                except (AttributeError, StoreManagementActionType.DoesNotExist):
                    pass
            try:
                request = RequestManager().get_request()
                if request:
                    messages.success(
                        request,
                        _('Recalculate {0} {1}').format(action_type.action_set.count(), action_type)
                    )
            except (RequestNotFound, AttributeError):
                pass
        return ret


class PricePolicyAdminForm(forms.ModelForm):
    """Assert than price policy is correct"""

    class Meta:
        model = PricePolicy
        fields = ('name', 'policy', 'parameters', 'apply_to', )

    def clean_parameters(self):
        parameters = self.cleaned_data['parameters']
        policy = self.cleaned_data['policy']
        if policy == 'multiply_purchase_by_ratio':
            try:
                Decimal(parameters)
            except InvalidOperation:
                text = ugettext('{0} is not a valid decimal'.format(parameters))
                if ',' in parameters:
                    text += ' ' + ugettext('Did you use coma rather than dot as decimal separator?')
                raise forms.ValidationError(text)
        return parameters


class StoreItemCategoryAdminForm(forms.ModelForm):
    """Assert than price policy is correct"""

    class Meta:
        model = StoreItemCategory
        fields = ('name', 'parent', 'price_policy', 'order_index', 'active', 'icon', 'default_image', )

    def clean_parent(self):
        parent = self.cleaned_data['parent']
        if self.instance and parent == self.instance:
            raise forms.ValidationError(ugettext('A category can not be its own parent'))

        if self.instance:
            if parent in self.instance.get_sub_categories():
                raise forms.ValidationError(
                    ugettext('A category can not be parent and children of another category')
                )

        return parent


class AddExtraSaleForm(BetterBsForm):
    """A form for adding a new extra sale"""

    analysis_code = forms.ChoiceField(label=_('Analysis code'), required=True)
    amount = forms.DecimalField(max_digits=9, decimal_places=2, required=True, label=_('Amount'))
    vat_rate = forms.ChoiceField(label=_('VAT rate'), required=True)
    date = forms.DateField(label=_('Date'), required=True)

    def __init__(self, *args, **kwargs):
        super(AddExtraSaleForm, self).__init__(*args, **kwargs)

        self.valid_action_types = [item.action_type for item in StoreManagementActionType.objects.all()]

        self.fields['analysis_code'].choices = [
            (analysis_code.id, analysis_code.name)
            for analysis_code in SaleAnalysisCode.objects.filter(action_type__in=self.valid_action_types)
        ]
        self.fields['vat_rate'].choices = [
            (vat_rate.id, vat_rate.name)
            for vat_rate in VatRate.objects.all()
        ]
        self.fields['date'].initial = date.today()

    def clean_analysis_code(self):
        """validate the analysis code"""
        analysis_code_id = self.cleaned_data['analysis_code']
        try:
            return SaleAnalysisCode.objects.get(id=analysis_code_id, action_type__in=self.valid_action_types)
        except SaleAnalysisCode.DoesNotExist:
            forms.ValidationError(ugettext('Invalid analysis code'))

    def clean_vat_rate(self):
        """validate the analysis code"""
        vat_rate_id = self.cleaned_data['vat_rate']
        try:
            return VatRate.objects.get(id=vat_rate_id)
        except VatRate.DoesNotExist:
            forms.ValidationError(ugettext('Invalid analysis code'))


class StockImportForm(BetterBsForm):
    """A form for importing stock values from Excel"""

    xls_file = forms.FileField(
        label=_('Excel file'),
        help_text=_('Same format than the export file. It will update the purchase price, stock count and threshold')
    )


class DateRangeForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        today = date.today()
        first_day_of_month = date(today.year, today.month, 1)
        if today.month == 12:
            month_of_next = 1
            year_of_next = today.year + 1
        else:
            month_of_next = today.month + 1
            year_of_next = today.year
        first_day_of_next_month = date(year_of_next, month_of_next, 1)
        last_day_of_month = first_day_of_next_month - timedelta(days=1)

        self.fields['start_date'] = forms.DateField(label=_("From"), required=True, initial=first_day_of_month)
        self.fields['end_date'] = forms.DateField(label=_("To"), required=True, initial=last_day_of_month)



# -*- coding: utf-8 -*-
"""test we can search contact by entity"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from model_mommy import mommy

from balafon.Crm import models

from balafon.Search.tests import BaseTestCase


class EntitySearchTest(BaseTestCase):
    """search by entity fields"""

    def test_search_entity(self):
        """search by entity name"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        url = reverse('search')

        data = {"gr0-_-entity_name-_-0": 'tiny'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_entity_description(self):
        """search by entity description"""
        entity1 = mommy.make(models.Entity, name="My tiny corp", description="This one should be found.")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp", description="Not this one")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="The big Org", description="Found")
        contact4 = mommy.make(models.Contact, entity=entity3, lastname="ABCABC", main_contact=True, has_left=False)

        url = reverse('search')

        data = {"gr0-_-entity_description-_-0": 'found'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertContains(response, entity3.name)
        self.assertContains(response, contact4.lastname)

    def test_search_entity_notes(self):
        """search by entity notes"""
        entity1 = mommy.make(models.Entity, name="My tiny corp", notes="This one should be found.")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp", notes="Not this one")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        entity3 = mommy.make(models.Entity, name="The big Org", notes="Found")
        contact4 = mommy.make(models.Contact, entity=entity3, lastname="ABCABC", main_contact=True, has_left=False)

        url = reverse('search')

        data = {"gr0-_-entity_notes-_-0": 'found'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

        self.assertContains(response, entity3.name)
        self.assertContains(response, contact4.lastname)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0006_auto_20160201_1025'),
    ]

    operations = [
        migrations.AddField(
            model_name='contact',
            name='same_as_priority',
            field=models.IntegerField(default=0, verbose_name='same as priority'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='same_as',
            field=models.ForeignKey(default=None, blank=True, to='Crm.SameAs', null=True, verbose_name='same as'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-03-28 13:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Emailing', '0003_auto_20180409_1301'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailing',
            name='lang',
            field=models.CharField(blank=True, default='', max_length=5, verbose_name='language'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Crm is the main module"""

from __future__ import unicode_literals

import django.dispatch

action_cloned = django.dispatch.Signal(providing_args=["original_action", "new_action"])

new_subscription = django.dispatch.Signal(providing_args=["instance", "contact"])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Users', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='custommenuitem',
            name='reverse_kwargs',
            field=models.CharField(default=b'', help_text='kwargs to use for building the reverse. kw1:defaultval1,kw2,kw3:defaultval3 ', max_length=100, verbose_name='reverse_kwargs', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0018_auto_20180104_1107'),
    ]

    operations = [
        migrations.CreateModel(
            name='ActionStatusTrack',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('datetime', models.DateTimeField()),
            ],
            options={
                'ordering': ('-datetime',),
                'verbose_name': 'Action state track',
                'verbose_name_plural': 'Action state tracks',
            },
        ),
        migrations.AddField(
            model_name='action',
            name='previous_status',
            field=models.ForeignKey(related_name='+', default=None, blank=True, to='Crm.ActionStatus', null=True),
        ),
        migrations.AddField(
            model_name='actiontype',
            name='track_status',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='actionstatustrack',
            name='action',
            field=models.ForeignKey(to='Crm.Action'),
        ),
        migrations.AddField(
            model_name='actionstatustrack',
            name='status',
            field=models.ForeignKey(to='Crm.ActionStatus'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('coop_cms', '0002_auto_20160108_1628'),
    ]

    operations = [
        migrations.CreateModel(
            name='CategoryPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('can_edit_groups', models.ManyToManyField(default=None, related_name='can_edit_perm', null=True, to='Crm.Group', blank=True)),
                ('can_view_groups', models.ManyToManyField(default=None, related_name='can_view_perm', null=True, to='Crm.Group', blank=True)),
                ('category', models.OneToOneField(to='coop_cms.ArticleCategory')),
            ],
        ),
        migrations.CreateModel(
            name='ContactProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('entity_name', models.CharField(default=b'', max_length=200, verbose_name='Entity name', blank=True)),
                ('zip_code', models.CharField(default='', max_length=20, verbose_name='Zip code', blank=True)),
                ('gender', models.IntegerField(default=0, blank=True, verbose_name='Gender', choices=[(1, 'Mr'), (2, 'Mrs')])),
                ('lastname', models.CharField(default='', max_length=200, verbose_name='last name', blank=True)),
                ('firstname', models.CharField(default='', max_length=200, verbose_name='first name', blank=True)),
                ('birth_date', models.DateField(default=None, null=True, verbose_name='birth date', blank=True)),
                ('phone', models.CharField(default='', max_length=200, verbose_name='phone', blank=True)),
                ('mobile', models.CharField(default='', max_length=200, verbose_name='mobile', blank=True)),
                ('address', models.CharField(default='', max_length=200, verbose_name='address', blank=True)),
                ('address2', models.CharField(default='', max_length=200, verbose_name='address 2', blank=True)),
                ('address3', models.CharField(default='', max_length=200, verbose_name='address 3', blank=True)),
                ('cedex', models.CharField(default='', max_length=200, verbose_name='cedex', blank=True)),
                ('subscriptions_ids', models.CharField(default=b'', max_length=100, blank=True)),
                ('city', models.ForeignKey(default=None, blank=True, to='Crm.City', null=True, verbose_name='City')),
                ('contact', models.OneToOneField(null=True, default=None, blank=True, to='Crm.Contact')),
                ('entity_type', models.ForeignKey(default=None, blank=True, to='Crm.EntityType', null=True, verbose_name='Entity type')),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-06 11:48
from __future__ import unicode_literals

import balafon.utils
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0020_auto_20180308_1558'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='actionmenu',
            options={'ordering': ['order_index'], 'verbose_name': 'action men', 'verbose_name_plural': 'action menus'},
        ),
        migrations.AlterField(
            model_name='action',
            name='detail',
            field=models.TextField(blank=True, default='', verbose_name='detail'),
        ),
        migrations.AlterField(
            model_name='action',
            name='subject',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='subject'),
        ),
        migrations.AlterField(
            model_name='action',
            name='uuid',
            field=models.CharField(blank=True, db_index=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='actiondocument',
            name='content',
            field=models.TextField(blank=True, default='', verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='actionmenu',
            name='a_attrs',
            field=models.CharField(blank=True, default='', help_text='Example: class="colorbox-form" for colorbos display', max_length=50, verbose_name='Link args'),
        ),
        migrations.AlterField(
            model_name='actionmenu',
            name='icon',
            field=models.CharField(blank=True, default='', max_length=30, verbose_name='icon'),
        ),
        migrations.AlterField(
            model_name='actionstatus',
            name='background_color',
            field=models.CharField(blank=True, default='', help_text='Background color. Must be a rgb code. For example: #000000', max_length=7, validators=[balafon.utils.validate_rgb], verbose_name='Background color'),
        ),
        migrations.AlterField(
            model_name='actionstatus',
            name='fore_color',
            field=models.CharField(blank=True, default='', help_text='Fore color. Must be a rgb code. For example: #ffffff', max_length=7, validators=[balafon.utils.validate_rgb], verbose_name='Fore color'),
        ),
        migrations.AlterField(
            model_name='actiontype',
            name='action_template',
            field=models.CharField(blank=True, default='', help_text='Action of this type will be displayed using the given template', max_length=200, verbose_name='action template'),
        ),
        migrations.AlterField(
            model_name='actiontype',
            name='default_template',
            field=models.CharField(blank=True, default='', help_text='Action of this type will have a document with the given template', max_length=200, verbose_name='document template'),
        ),
        migrations.AlterField(
            model_name='actiontype',
            name='mail_to_subject',
            field=models.CharField(blank=True, default='', help_text='This would be used as subject when sending the action by email', max_length=100, verbose_name='Subject of email'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='billing_street_number',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='street number'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='favorite_language',
            field=models.CharField(blank=True, choices=[('', 'Default'), ('en', 'English'), ('fr', 'Français')], default='', max_length=10, verbose_name='favorite language'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='gender',
            field=models.IntegerField(blank=True, choices=[(0, ''), (1, 'Mr'), (2, 'Mrs'), (3, 'Mrs and Mr')], default=0, verbose_name='gender'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='notes',
            field=models.TextField(blank=True, default='', verbose_name='notes'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='street_number',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='street number'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='uuid',
            field=models.CharField(blank=True, db_index=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='contactsimport',
            name='encoding',
            field=models.CharField(choices=[('utf-8', 'utf-8'), ('iso-8859-15', 'iso-8859-15'), ('cp1252', 'cp1252')], default='utf-8', max_length=50),
        ),
        migrations.AlterField(
            model_name='contactsimport',
            name='separator',
            field=models.CharField(choices=[(',', 'Coma'), (';', 'Semi-colon')], default=',', max_length=5),
        ),
        migrations.AlterField(
            model_name='customfield',
            name='label',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='label'),
        ),
        migrations.AlterField(
            model_name='customfield',
            name='widget',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='widget'),
        ),
        migrations.AlterField(
            model_name='entity',
            name='billing_street_number',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='street number'),
        ),
        migrations.AlterField(
            model_name='entity',
            name='description',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='entity',
            name='notes',
            field=models.TextField(blank=True, default='', verbose_name='notes'),
        ),
        migrations.AlterField(
            model_name='entity',
            name='street_number',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='street number'),
        ),
        migrations.AlterField(
            model_name='entity',
            name='website',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='web site'),
        ),
        migrations.AlterField(
            model_name='group',
            name='background_color',
            field=models.CharField(blank=True, default='', help_text='Background color. Must be a rgb code. For example: #000000', max_length=7, validators=[balafon.utils.validate_rgb], verbose_name='Background color'),
        ),
        migrations.AlterField(
            model_name='group',
            name='description',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='group',
            name='fore_color',
            field=models.CharField(blank=True, default='', help_text='Fore color. Must be a rgb code. For example: #ffffff', max_length=7, validators=[balafon.utils.validate_rgb], verbose_name='Fore color'),
        ),
        migrations.AlterField(
            model_name='mailtosettings',
            name='body_template',
            field=models.TextField(blank=True, default='', verbose_name='body template'),
        ),
        migrations.AlterField(
            model_name='mailtosettings',
            name='subject',
            field=models.CharField(blank=True, default='', help_text='Use action subject if empty', max_length=100, verbose_name='subject'),
        ),
        migrations.AlterField(
            model_name='opportunity',
            name='detail',
            field=models.TextField(blank=True, default='', verbose_name='detail'),
        ),
        migrations.AlterField(
            model_name='opportunity',
            name='display_on_board',
            field=models.BooleanField(db_index=True, default=True, verbose_name='display on board'),
        ),
        migrations.AlterField(
            model_name='relationshiptype',
            name='reverse',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='reverse relation'),
        ),
        migrations.AlterField(
            model_name='zone',
            name='code',
            field=models.CharField(blank=True, default='', max_length=10, verbose_name='code'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Brand',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'verbose_name': 'Brand',
                'verbose_name_plural': 'Brands',
            },
        ),
        migrations.CreateModel(
            name='Certificate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('logo', models.ImageField(default=b'', upload_to=b'certificates', blank=True)),
            ],
            options={
                'verbose_name': 'certificate',
                'verbose_name_plural': 'certificates',
            },
        ),
        migrations.CreateModel(
            name='DeliveryPoint',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Delivery point',
                'verbose_name_plural': 'Delivery points',
            },
        ),
        migrations.CreateModel(
            name='Discount',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('short_name', models.CharField(default=b'', max_length=100, verbose_name='short name', blank=True)),
                ('quantity', models.DecimalField(default=0, verbose_name='quantity', max_digits=9, decimal_places=2)),
                ('rate', models.DecimalField(default=0, verbose_name='rate', max_digits=4, decimal_places=2)),
                ('active', models.BooleanField(default=False, help_text='Only active discounts are taken into account on a new purchase', verbose_name='active')),
            ],
            options={
                'ordering': ['rate', 'name'],
                'verbose_name': 'Discount',
                'verbose_name_plural': 'Discounts',
            },
        ),
        migrations.CreateModel(
            name='Favorite',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
            options={
                'verbose_name': 'Favorite',
                'verbose_name_plural': 'Favorites',
            },
        ),
        migrations.CreateModel(
            name='PriceClass',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('description', models.CharField(default=b'', max_length=100, verbose_name='Description', blank=True)),
                ('discounts', models.ManyToManyField(to='Store.Discount', verbose_name='Discount', blank=True)),
            ],
            options={
                'verbose_name': 'Price class',
                'verbose_name_plural': 'Price classes',
            },
        ),
        migrations.CreateModel(
            name='PricePolicy',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('parameters', models.CharField(default=b'', max_length=100, verbose_name='parameters', blank=True)),
                ('policy', models.CharField(max_length=100, verbose_name='policy', choices=[(b'from_category', 'Inherit from category'), (b'multiply_purchase_by_ratio', 'Multiply purchase price by ratio')])),
                ('apply_to', models.IntegerField(default=0, verbose_name='apply to', choices=[(0, 'All'), (1, 'Articles'), (2, 'Categories')])),
            ],
            options={
                'verbose_name': 'Price policy',
                'verbose_name_plural': 'Price policies',
            },
        ),
        migrations.CreateModel(
            name='Sale',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('action', models.OneToOneField(to='Crm.Action')),
                ('delivery_point', models.ForeignKey(default=None, blank=True, to='Store.DeliveryPoint', null=True, verbose_name='delivery point')),
            ],
            options={
                'verbose_name': 'Sale',
                'verbose_name_plural': 'Sales',
            },
        ),
        migrations.CreateModel(
            name='SaleItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('quantity', models.DecimalField(verbose_name='quantity', max_digits=9, decimal_places=2)),
                ('pre_tax_price', models.DecimalField(verbose_name='pre-tax price', max_digits=9, decimal_places=2)),
                ('text', models.TextField(default=b'', max_length=3000, verbose_name='Text', blank=True)),
                ('order_index', models.IntegerField(default=0)),
                ('is_blank', models.BooleanField(default=False, help_text='displayed as an empty line', verbose_name='is blank')),
                ('discount', models.ForeignKey(default=None, blank=True, to='Store.Discount', null=True)),
            ],
            options={
                'ordering': ['order_index'],
                'verbose_name': 'Sale item',
                'verbose_name_plural': 'Sales items',
            },
        ),
        migrations.CreateModel(
            name='StoreItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('pre_tax_price', models.DecimalField(verbose_name='pre-tax price', max_digits=9, decimal_places=2)),
                ('stock_count', models.DecimalField(decimal_places=2, default=None, max_digits=9, blank=True, null=True, verbose_name='stock count')),
                ('stock_threshold', models.DecimalField(decimal_places=2, default=None, max_digits=9, blank=True, null=True, verbose_name='stock threshold')),
                ('purchase_price', models.DecimalField(decimal_places=2, default=None, max_digits=9, blank=True, null=True, verbose_name='purchase price')),
                ('reference', models.CharField(default=b'', max_length=100, verbose_name='reference', blank=True)),
                ('available', models.BooleanField(default=True, verbose_name='Available')),
                ('brand', models.ForeignKey(default=None, blank=True, to='Store.Brand', null=True, verbose_name='brand')),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'Store item',
                'verbose_name_plural': 'Store items',
            },
        ),
        migrations.CreateModel(
            name='StoreItemCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('order_index', models.IntegerField(default=0, verbose_name='order_index')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
                ('icon', models.CharField(default=b'', max_length=20, blank=True)),
                ('parent', models.ForeignKey(related_name='subcategories_set', default=None, blank=True, to='Store.StoreItemCategory', null=True, verbose_name='parent category')),
                ('price_policy', models.ForeignKey(default=None, blank=True, to='Store.PricePolicy', null=True, verbose_name='price policy')),
            ],
            options={
                'ordering': ['order_index', 'name'],
                'verbose_name': 'Store item category',
                'verbose_name_plural': 'Store item categories',
            },
        ),
        migrations.CreateModel(
            name='StoreItemImport',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('data', models.FileField(help_text='An Excel file (*.xls) with the data', upload_to=b'store_item_imports', verbose_name='Import file')),
                ('fields', models.CharField(default=b'name,brand,reference,category,purchase_price,vat_rate', help_text="Fields to import in order: if the attribute doesn't exist, create a custom property", max_length=300, verbose_name='fields')),
                ('last_import_date', models.DateTimeField(default=None, null=True, verbose_name='last import', blank=True)),
                ('import_error', models.TextField(default=b'', verbose_name='import error', blank=True)),
                ('is_successful', models.BooleanField(default=True, verbose_name='is successful')),
                ('ignore_first_line', models.BooleanField(default=True, verbose_name='ignore first line')),
                ('margin_rate', models.DecimalField(default=None, null=True, verbose_name='margin_rate', max_digits=9, decimal_places=2)),
                ('error_message', models.CharField(default=b'', max_length=100, verbose_name='error message', blank=True)),
                ('category_lines_mode', models.BooleanField(default=False, help_text='If checked, the store item category are expected as merged-cells header lines', verbose_name='category-lines mode')),
                ('default_brand', models.CharField(default=b'', help_text='If defined, it will be used if no brand is given', max_length=50, verbose_name='default Brand', blank=True)),
                ('make_available', models.BooleanField(default=False, verbose_name='articles will be available if price is set')),
            ],
            options={
                'verbose_name': 'Store item import',
                'verbose_name_plural': 'Store item imports',
            },
        ),
        migrations.CreateModel(
            name='StoreItemProperty',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('label', models.CharField(default=b'', max_length=100, verbose_name='label', blank=True)),
                ('in_fullname', models.BooleanField(default=False, help_text='is inserted in fullname if checked', verbose_name='in fullname')),
                ('is_public', models.BooleanField(default=False, help_text='returned in public API', verbose_name='is public')),
            ],
            options={
                'verbose_name': 'Store item: property value',
                'verbose_name_plural': 'Store item: properties',
            },
        ),
        migrations.CreateModel(
            name='StoreItemPropertyValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.CharField(default=b'', max_length=100, verbose_name='value', blank=True)),
                ('item', models.ForeignKey(verbose_name='item', to='Store.StoreItem')),
                ('property', models.ForeignKey(verbose_name='property', to='Store.StoreItemProperty')),
            ],
            options={
                'verbose_name': 'Store item: property value',
                'verbose_name_plural': 'Store item: property values',
            },
        ),
        migrations.CreateModel(
            name='StoreItemTag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('order_index', models.IntegerField(default=0, verbose_name='order_index')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
                ('icon', models.CharField(default=b'', max_length=20, blank=True)),
                ('show_always', models.BooleanField(default=True, verbose_name='show always')),
            ],
            options={
                'ordering': ['order_index', 'name'],
                'verbose_name': 'Store item tag',
                'verbose_name_plural': 'Store item tags',
            },
        ),
        migrations.CreateModel(
            name='StoreManagementActionType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('template_name', models.CharField(default=b'', help_text='Set the name of a custom template for commercial document', max_length=100, verbose_name='template name', blank=True)),
                ('show_amount_as_pre_tax', models.BooleanField(default=True, help_text='The action amount will be update with pre-tax total if checked and with tax-included total if not', verbose_name='Show amount as pre-tax')),
                ('action_type', models.OneToOneField(to='Crm.ActionType')),
                ('readonly_status', models.ManyToManyField(help_text='When action has one of these status, it is not possible to modify a commercial document', to='Crm.ActionStatus', verbose_name='readonly status', blank=True)),
            ],
            options={
                'verbose_name': 'Store management action type',
                'verbose_name_plural': 'Store management action types',
            },
        ),
        migrations.CreateModel(
            name='Supplier',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Supplier',
                'verbose_name_plural': 'Suppliers',
            },
        ),
        migrations.CreateModel(
            name='Unit',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
            ],
            options={
                'verbose_name': 'unit',
                'verbose_name_plural': 'units',
            },
        ),
        migrations.CreateModel(
            name='VatRate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('rate', models.DecimalField(verbose_name='vat rate', max_digits=4, decimal_places=2)),
                ('is_default', models.BooleanField(default=False, verbose_name='is default')),
            ],
            options={
                'ordering': ['rate'],
                'verbose_name': 'VAT rate',
                'verbose_name_plural': 'VAT rates',
            },
        ),
        migrations.AddField(
            model_name='storeitemimport',
            name='supplier',
            field=models.ForeignKey(default=None, blank=True, to='Store.Supplier', null=True, verbose_name='Supplier'),
        ),
        migrations.AddField(
            model_name='storeitemimport',
            name='tags',
            field=models.ManyToManyField(help_text='Tags that can be added to the items', to='Store.StoreItemTag', verbose_name='tags', blank=True),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='category',
            field=models.ForeignKey(verbose_name='category', to='Store.StoreItemCategory'),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='certificates',
            field=models.ManyToManyField(to='Store.Certificate', verbose_name='certificate', blank=True),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='imported_by',
            field=models.ForeignKey(default=None, blank=True, to='Store.StoreItemImport', null=True, verbose_name='imported by'),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='price_class',
            field=models.ForeignKey(default=None, blank=True, to='Store.PriceClass', null=True, verbose_name='price class'),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='price_policy',
            field=models.ForeignKey(default=None, blank=True, to='Store.PricePolicy', null=True, verbose_name='price policy'),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='supplier',
            field=models.ForeignKey(default=None, blank=True, to='Store.Supplier', null=True, verbose_name='Supplier'),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='tags',
            field=models.ManyToManyField(to='Store.StoreItemTag', verbose_name='tags', blank=True),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='unit',
            field=models.ForeignKey(default=None, blank=True, to='Store.Unit', null=True),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='vat_rate',
            field=models.ForeignKey(verbose_name='VAT rate', to='Store.VatRate'),
        ),
        migrations.AddField(
            model_name='saleitem',
            name='item',
            field=models.ForeignKey(default=None, blank=True, to='Store.StoreItem', null=True),
        ),
        migrations.AddField(
            model_name='saleitem',
            name='sale',
            field=models.ForeignKey(to='Store.Sale'),
        ),
        migrations.AddField(
            model_name='saleitem',
            name='vat_rate',
            field=models.ForeignKey(default=None, blank=True, to='Store.VatRate', null=True, verbose_name='VAT rate'),
        ),
        migrations.AddField(
            model_name='favorite',
            name='item',
            field=models.ForeignKey(verbose_name='store item', to='Store.StoreItem'),
        ),
        migrations.AddField(
            model_name='favorite',
            name='user',
            field=models.ForeignKey(verbose_name='user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='discount',
            name='tags',
            field=models.ManyToManyField(to='Store.StoreItemTag', verbose_name='tags', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib import admin
from balafon.Users import models
from balafon.Users.forms import UserPreferencesAdminForm, UserPermissionsAdminForm


class UserPreferencesAdmin(admin.ModelAdmin):
    form = UserPreferencesAdminForm
    list_display = ['user', 'notify_due_actions', 'message_in_favorites', ]

admin.site.register(models.UserPreferences, UserPreferencesAdmin)


class UserPermissionsAdmin(admin.ModelAdmin):
    form = UserPermissionsAdminForm
    list_display = ['user', 'can_create_group', ]
    list_editable = ['can_create_group', ]

admin.site.register(models.UserPermissions, UserPermissionsAdmin)


class FavoriteAdmin(admin.ModelAdmin):
    list_display = ["__str__", "user", "content_type", "object_id"]
    list_filter = ["user", "content_type"]

admin.site.register(models.Favorite, FavoriteAdmin)


class UserHomepageAdmin(admin.ModelAdmin):
    list_display = ["user", "url"]

admin.site.register(models.UserHomepage, UserHomepageAdmin)


class CustomMenuItemInline(admin.TabularInline):
    model = models.CustomMenuItem


class CustomMenuAdmin(admin.ModelAdmin):
    inlines = [CustomMenuItemInline]

admin.site.register(models.CustomMenu, CustomMenuAdmin)


# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import sys

from django.conf import settings
from django.conf.urls import include, url
from django.contrib.auth import views as django_auth_views
from django.contrib.staticfiles.views import serve as serve_static
from django.views.static import serve as serve_media

from coop_cms.settings import get_url_patterns, get_media_root

from balafon.forms import BsAuthenticationForm, BsPasswordChangeForm, BsPasswordResetForm
from balafon.Users import views as users_views
from balafon.views import redirect_to_homepage, auto_save_data


if getattr(settings, 'BALAFON_NOTIFY_SUBSCRIPTIONS', ''):
    raise Exception("Invalid setting : BALAFON_NOTIFY_SUBSCRIPTIONS has been replaced by BALAFON_NOTIFICATION_EMAIL")


localized_patterns = get_url_patterns()

urlpatterns = [
    url(r'^crm/$', users_views.user_homepage, name="balafon_homepage"),
]


if settings.DEBUG or ('test' in sys.argv) or getattr(settings, 'SERVE_STATIC', True):
    if settings.DEBUG:
        urlpatterns += [
            url(r'^static/(?P<path>.*)$', serve_static),
        ]
    else:
        urlpatterns += [
            url(r'^static/(?P<path>.*)$', serve_media, {'document_root': settings.STATIC_ROOT}),
        ]
    urlpatterns += [
        url(
            r'^media/(?P<path>.*)$',
            serve_media,
            {'document_root': get_media_root(), 'show_indexes': True}
        ),
    ]


urlpatterns += localized_patterns(
    url(r'^crm/', include('balafon.Crm.urls')),
    url(r'^crm/', include('balafon.Crm.api_urls')),
    url(r'^crm-search/', include('balafon.Search.urls')),
    url('^crm/go-to-home/', redirect_to_homepage, name="homepage"),
    url(
        r'^auto-save/(?P<model_type>\w+)/(?P<field_name>[\w-]+)/(?P<obj_id>\d+)/$',
        auto_save_data,
        name="auto_save_data"
    ),
    url(r'^emailing/', include('balafon.Emailing.urls')),
)

if 'coop_cms.apps.email_auth' in settings.INSTALLED_APPS:
    urlpatterns += localized_patterns(
        url(r'^accounts/', include('coop_cms.apps.email_auth.urls')),
    )

    if 'registration' in settings.INSTALLED_APPS and 'balafon.Profile' not in settings.INSTALLED_APPS:
        urlpatterns += localized_patterns(
            url(r'^accounts/', include('coop_cms.apps.email_auth.registration_backend.urls')),
        )

else:

    urlpatterns += localized_patterns(
        url(
            r'^accounts/login/$',
            django_auth_views.login,
            {'authentication_form': BsAuthenticationForm},
            name='login'
        ),
        url(r'^accounts/password_change/$',
            django_auth_views.password_change,
            {'password_change_form': BsPasswordChangeForm},
            name='password_change'
        ),
        url(
            r'^accounts/password_reset/$',
            django_auth_views.password_reset,
            {'password_reset_form': BsPasswordResetForm},
            name='password_reset'
        ),
    )


urlpatterns += localized_patterns(
    url(r'^accounts/', include('django.contrib.auth.urls'))
)



if 'balafon.Profile' in settings.INSTALLED_APPS:
    urlpatterns += localized_patterns(
        url(r'^accounts/', include('balafon.Profile.urls'))
    )


if 'balafon.Store' in settings.INSTALLED_APPS:
    urlpatterns += localized_patterns(
        url(r'^store/', include('balafon.Store.urls')),
        url(r'^store/', include('balafon.Store.api.urls'))
    )

if 'balafon.Users' in settings.INSTALLED_APPS:
    urlpatterns += localized_patterns(
        url(r'^users/', include('balafon.Users.urls')),
    )


if getattr(settings, 'BALAFON_AS_HOMEPAGE', False):
    urlpatterns += [
        url(r'^$', users_views.user_homepage, name='homepage'),
    ]


urlpatterns += localized_patterns(
    url(r'^', include('coop_cms.urls')),
)



# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

from bs4 import BeautifulSoup
from datetime import datetime, timedelta
from importlib import import_module
import re

from django.conf import settings
from django.core.exceptions import ValidationError
from django.contrib.sites.models import Site
from django.utils.timezone import now as dt_now
from django.utils.translation import ugettext, ugettext_lazy as _

from coop_cms.models import Newsletter
from coop_cms.settings import get_newsletter_templates
from coop_cms.utils import dehtml
from coop_cms.bs_forms import Form as BsForm, ModelForm as BsModelForm
import floppyforms.__future__ as forms

from balafon.moves import urlopen
from balafon.settings import get_captcha_field
from balafon.Crm import settings as crm_settings
from balafon.Crm.forms import ModelFormWithCity, BetterBsModelForm
from balafon.Crm.models import Group, Contact, Entity, EntityType, Action, ActionType, SubscriptionType, Subscription
from balafon.Crm.widgets import CityAutoComplete
from balafon.Emailing import models
from balafon.Emailing.utils import create_subscription_action, send_notification_email, get_language
from balafon.Emailing import settings as emailing_settings


class UnregisterForm(BsForm):
    """User wants to unregister from emailing"""
    reason = forms.CharField(required=False, widget=forms.Textarea, label=_("Reason"))


class UpdateEmailingForm(BsModelForm):
    """form for editing an existing emailing"""

    class Meta:
        model = models.Emailing
        fields = ('subscription_type', 'newsletter', 'lang', 'from_email', )

    def __init__(self, *args, **kwargs):
        super(UpdateEmailingForm, self).__init__(*args, **kwargs)

        subscription_choices = [(subs_type.id, subs_type.name) for subs_type in SubscriptionType.objects.all()]
        self.fields["subscription_type"].widget = forms.Select(choices=subscription_choices)

        newsletter_choices = [(newsletter.id, newsletter.subject) for newsletter in Newsletter.objects.all()]
        self.fields["newsletter"].widget = forms.Select(choices=newsletter_choices, attrs={'class': 'form-control'})

        if not getattr(settings, 'BALAFON_EMAILING_SENDER_CHOICES', None):
            self.fields["from_email"].widget = forms.HiddenInput()
        else:
            self.fields["from_email"].widget = forms.Select(choices=settings.BALAFON_EMAILING_SENDER_CHOICES)

        if not getattr(settings, 'LANGUAGES', None) or len(settings.LANGUAGES) < 2:
            self.fields["lang"].widget = forms.HiddenInput()
        else:
            language_choices = crm_settings.get_language_choices(_("Favorite language of the contact"))
            self.fields["lang"].widget = forms.Select(choices=language_choices, attrs={'class': 'form-control'})


class NewEmailingForm(BsForm):
    """Form for creating a new emailing"""

    subscription_type = forms.IntegerField(label=_("Subscription Type"))
    newsletter = forms.IntegerField(label=_("Newsletter"))
    subject = forms.CharField(
        label=_("Subject"), required=False,
        widget=forms.TextInput(attrs={'placeholder': _('Subject of the newsletter')})
    )
    contacts = forms.CharField(widget=forms.HiddenInput())
    lang = forms.CharField(
        required=False,
        label=_("Language"),
        widget=forms.Select(choices=[('', _('Default'))] + list(settings.LANGUAGES))
    )
    from_email = forms.CharField(required=False, label=_("Sent from"))

    def __init__(self, *args, **kwargs):
        initial = kwargs.get('initial')
        initial_contacts = ''
        if initial and 'contacts' in initial:
            initial_contacts = ';'.join(['{0}'.format(contact.id) for contact in initial['contacts']])
            initial.pop('contacts')
        super(NewEmailingForm, self).__init__(*args, **kwargs)
        if initial_contacts:
            self.fields['contacts'].initial = initial_contacts

        newsletter_choices = [
            (0, ugettext('-- New --'))
        ] + [
            (newsletter.id, newsletter.subject) for newsletter in Newsletter.objects.all().order_by('-id')
        ]
        self.fields["newsletter"].widget = forms.Select(choices=newsletter_choices, attrs={'class': 'form-control'})

        subscription_choices = [(subscription.id, subscription.name) for subscription in SubscriptionType.objects.all()]
        self.fields["subscription_type"].widget = forms.Select(
            choices=subscription_choices, attrs={'class': 'form-control'}
        )

        if not getattr(settings, 'LANGUAGES', None) or len(settings.LANGUAGES) < 2:
            self.fields["lang"].widget = forms.HiddenInput()
        else:
            language_choices = crm_settings.get_language_choices(_("Favorite language of the contact"))
            self.fields["lang"].widget = forms.Select(choices=language_choices, attrs={'class': 'form-control'})

        if getattr(settings, 'BALAFON_EMAILING_SENDER_CHOICES', None):
            self.fields['from_email'].widget = forms.Select(
                choices=settings.BALAFON_EMAILING_SENDER_CHOICES, attrs={'class': 'form-control'}
            )
        else:
            self.fields['from_email'].widget = forms.HiddenInput()

    def get_contacts(self):
        """get the list of contacts stored by ids"""
        ids = self.cleaned_data["contacts"].split(";")
        return models.Contact.objects.filter(id__in=ids)

    def clean_subject(self):
        """subject validation"""
        newsletter_id = int(self.cleaned_data['newsletter'])
        subject = self.cleaned_data['subject']
        if newsletter_id == 0 and not subject:
            raise ValidationError(ugettext("Please enter a subject for the newsletter"))
        return subject

    def clean_subscription_type(self):
        """validation of subscription type. Return the subscription type object"""
        try:
            subscription_type = int(self.cleaned_data['subscription_type'])
            return SubscriptionType.objects.get(id=subscription_type)
        except (ValueError, KeyError, SubscriptionType.DoesNotExist):
            raise ValidationError(ugettext("Please select a valid subscription"))


class NewNewsletterForm(BsForm):
    """Create a new newsletter"""
    subject = forms.CharField(
        label=_("Subject"),
        widget=forms.TextInput(attrs={'placeholder': _('Subject of the newsletter')})
    )
    template = forms.ChoiceField(label=_("Template"), choices=get_newsletter_templates(None, None))
    source_url = forms.URLField(label=_('Source URL'), required=False)
    content = forms.CharField(label=_('Content'), required=False, widget=forms.HiddenInput())
    
    def __init__(self, *args, **kwargs):
        super(NewNewsletterForm, self).__init__(*args, **kwargs)
        
        self.source_content = ""
        
        self.allow_url_sources = getattr(settings, 'BALAFON_NEWSLETTER_SOURCES', ())
        if not self.allow_url_sources:
            self.fields['source_url'].widget = forms.HiddenInput()
    
    def clean_source_url(self):
        """
        clean source url: If an url is given download newsletter content from this source
        It can be used for people using a different tool for writing their newsletter
        """
        url = self.cleaned_data['source_url']
        if url:
            # Check in config if the url match with something allowed
            newsletter_sources = getattr(settings, 'BALAFON_NEWSLETTER_SOURCES', ())
            for (regex, selector, post_processor) in newsletter_sources:
                if re.match(regex, url):
                    try:
                        # if so get the content
                        html = urlopen(url)
                        # and extract the selector content as initial content for the newsletter
                        soup = BeautifulSoup(html, "html.parser")
                        content = ''.join(['{0}'.format(tag) for tag in soup.select(selector)])
                        if post_processor:
                            # import the post_processor function
                            module_name, processor_name = post_processor.rsplit('.', 1)
                            module = import_module(module_name)
                            post_processor_func = getattr(module, processor_name)
                            # call the post_processor and update the content
                            content = post_processor_func(content)
                        self.source_content = content
                        return url
                    except Exception as msg:
                        raise ValidationError(msg)
            raise ValidationError(ugettext("The url is not allowed"))
        return ''

    def clean_content(self):
        """content validation"""
        url = self.cleaned_data['source_url']
        if url:
            return self.source_content
        return ''


class EmailSubscribeForm(BetterBsModelForm):
    """Register to an emailing with just email address"""
    email = forms.EmailField(
        required=True, label="",
        widget=forms.TextInput(attrs={'placeholder': _('Email'), 'size': '80'})
    )
    
    class Meta:
        model = Contact
        fields = ('email', 'favorite_language')

    def __init__(self, *args, **kwargs):
        self.subscription_type = kwargs.pop('subscription_type', None)
        super(EmailSubscribeForm, self).__init__(*args, **kwargs)

        self.fields['favorite_language'].widget = forms.HiddenInput()
        if crm_settings.has_language_choices():
            self.fields['favorite_language'].initial = get_language()
    
    def save(self, request=None):
        """save"""
        contact = super(EmailSubscribeForm, self).save(commit=False)
        
        if crm_settings.ALLOW_SINGLE_CONTACT:
            contact.entity = Entity.objects.create(name=contact.email, type=None, is_single_contact=True)
        else:
            et_id = getattr(settings, 'BALAFON_INDIVIDUAL_ENTITY_ID', 1)
            entity_type = EntityType.objects.get(id=et_id)
            contact.entity = Entity.objects.create(name=contact.email, type=entity_type)
        contact.save()
        # delete unknown contacts for the current entity
        contact.entity.contact_set.exclude(id=contact.id).delete()

        queryset = SubscriptionType.objects.filter(site=Site.objects.get_current())

        form_subscription_type = self.subscription_type
        default_subscription_type = emailing_settings.get_default_subscription_type()
        if not form_subscription_type and default_subscription_type:
            form_subscription_type = default_subscription_type

        if form_subscription_type:
            queryset = queryset.filter(id=form_subscription_type)

        subscriptions = []
        for subscription_type in queryset:
            subscription = Subscription.objects.get_or_create(contact=contact, subscription_type=subscription_type)[0]
            subscription.accept_subscription = True
            subscription.subscription_date = datetime.now()
            subscription.save()
            subscriptions.append(subscription_type.name)
        
        create_subscription_action(contact, subscriptions)
        if subscriptions:
            send_notification_email(request, contact, [], "")
        else:
            send_notification_email(request, contact, [], "Error: "+ugettext("No subscription_type defined"))
                
        return contact


class SubscriptionTypeFormMixin(object):
    """Base class requiring subscription type"""
    subscription_required = False
    subscription_types_count = 0

    def _add_subscription_types_field(self, contact=None):
        """add the subscription_type field dynamically"""
        subscription_types = list(self.get_queryset())
        self.subscription_types_count = len(subscription_types)
        if self.subscription_types_count > 0:
            if self.subscription_types_count > 1:
                help_text = ugettext('Check the boxes of the newsletters that you want to receive')
            else:
                help_text = ugettext('Check the box if you want to receive our newsletter')

            initial = None
            if contact:
                initial = ','.join(
                    [
                        str(subscription.subscription_type.id)
                        for subscription in contact.subscription_set.all()
                        if subscription.accept_subscription
                    ]
                )

            self.fields['subscription_types'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='',
                help_text=help_text,
                required=self.subscription_required and (self.subscription_types_count == 1),
                initial=initial,
                choices=[(subscription_type.id, subscription_type.name) for subscription_type in subscription_types]
            )

        else:
            self.fields['subscription_types'] = forms.MultipleChoiceField(
                widget=forms.HiddenInput(),
                label='',
                required=False,
                choices=[]
            )

    def get_queryset(self):
        """returns subscription_types"""
        return SubscriptionType.objects.filter(site=Site.objects.get_current())

    def clean_subscription_types(self):
        """validation"""
        try:
            subscription_types = [
                SubscriptionType.objects.get(id=st_id) for st_id in self.cleaned_data['subscription_types']
            ]
        except SubscriptionType.DoesNotExist:
            raise ValidationError(_("Invalid subscription type"))

        if self.subscription_required and not (len(subscription_types)):
            if self.subscription_types_count > 1:
                raise ValidationError(_("You must check a least one subscription"))
            else:
                raise ValidationError(_("This field is required"))
        return subscription_types

    def _save_subscription_types(self, contact):
        """save"""
        subscriptions = []
        for subscription_type in self.get_queryset():

            subscription = Subscription.objects.get_or_create(
                contact=contact,
                subscription_type=subscription_type
            )[0]

            if subscription_type in self.cleaned_data['subscription_types']:
                subscription.accept_subscription = True
                subscription.subscription_date = datetime.now()
                #This is added to the notification email
                subscriptions.append(subscription_type.name)
            else:
                subscription.accept_subscription = False
            subscription.save()
        return subscriptions


class SubscribeForm(ModelFormWithCity, SubscriptionTypeFormMixin):
    """Subscribe to emailing"""

    city = forms.CharField(
        required=False, label=_('City'),
        widget=CityAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
    )
    entity_type = forms.ChoiceField(required=False, widget=forms.Select())
    entity = forms.CharField(
        required=False,
        widget=forms.TextInput(attrs={'placeholder': _('Name of the entity')})
    )
    groups = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple(), label='', required=False)
    action_types = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple(), label='', required=False)
    message = forms.CharField(required=False, widget=forms.Textarea(attrs={'placeholder': _('Message'), 'cols':'90'}))
    captcha = get_captcha_field()
    favorite_language = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = Contact
        fields = (
            'gender', 'firstname', 'lastname', 'phone', 'mobile', 'email', 'address',
            'address2', 'address3', 'zip_code'
        )
        widgets = {
            'lastname': forms.TextInput(attrs={'placeholder': _('Lastname'), 'required': 'required'}),
            'firstname': forms.TextInput(attrs={'placeholder': _('Firstname')}),
            'phone': forms.TextInput(attrs={'placeholder': _('Phone')}),
            'email': forms.TextInput(attrs={'placeholder': _('Email'), 'required': 'required'}),
            'zip_code': forms.TextInput(attrs={'placeholder': _('zip code')}),
        }

    def __init__(self, *args, **kwargs):
        super(SubscribeForm, self).__init__(*args, **kwargs)

        self.fields['email'].required = True
        #self.fields['lastname'].required = True
        
        # Do not display (Mrs and M) gender on subscribe form
        self.fields['gender'].choices = self.fields['gender'].choices[:3]

        entity_types_choices = []

        if crm_settings.ALLOW_SINGLE_CONTACT:
            entity_types_choices.append((0, _('Individual')))
        else:
            entity_types_choices.append((0, ''))

        entity_types_choices.extend([
            (et.id, et.name) for et in EntityType.objects.filter(subscribe_form=True)
        ])

        self.fields['entity_type'].choices = entity_types_choices

        self.fields['groups'].choices = [
            (group.id, group.name) for group in Group.objects.filter(subscribe_form=True)
        ]
        
        self.fields['action_types'].choices = [
            (action_type.id, action_type.name) for action_type in ActionType.objects.filter(subscribe_form=True)
        ]
        
        self._add_subscription_types_field()

        if crm_settings.has_language_choices():
            self.fields['favorite_language'].initial = get_language()
    
    def clean_entity_type(self):
        """validation"""
        try:
            entity_type = int(self.cleaned_data['entity_type'])
            if entity_type:
                return EntityType.objects.get(id=entity_type)
            return None
        except (ValueError, EntityType.DoesNotExist):
            raise ValidationError(ugettext("Invalid entity type"))
        
    def get_entity(self):
        """get entity from form"""
        entity_type = self.cleaned_data.get('entity_type', None)
        entity = self.cleaned_data['entity']
        if entity_type:
            if entity:
                return Entity.objects.create(name=entity, type=entity_type)
        else:
            if crm_settings.ALLOW_SINGLE_CONTACT:
                return Entity.objects.create(name=entity, type=None, is_single_contact=True)
            else:
                et_id = getattr(settings, 'BALAFON_INDIVIDUAL_ENTITY_ID', 1)
                entity_type = EntityType.objects.get(id=et_id)
                entity_name = "{0} {1}".format(
                    self.cleaned_data['lastname'], self.cleaned_data['firstname'])
                return Entity.objects.create(name=entity_name, type=entity_type)
            
    def clean_entity(self):
        """validation"""
        entity_type = self.cleaned_data.get('entity_type', None)
        entity = self._dehtmled_field("entity")
        if entity_type:
            if not entity:
                raise ValidationError("{0}: {1}".format(entity_type.name, ugettext("Please enter a name")))
        else:
            data = [self.cleaned_data[x] for x in ('lastname', 'firstname')]
            entity = ' '.join([x for x in data if x]).strip().upper()
            
        return entity
         
    def _dehtmled_field(self, fieldname, **kwargs):
        """html to text for a field content"""
        value = self.cleaned_data[fieldname]
        return dehtml(value, **kwargs)
        
    def clean_lastname(self):
        """validate lastname"""
        return self._dehtmled_field("lastname")
    
    def clean_firstname(self):
        """validate firstname"""
        return self._dehtmled_field("firstname")
    
    def clean_phone(self):
        """validate phone"""
        return self._dehtmled_field("phone")
    
    def clean_mobile(self):
        """validate mobile phone"""
        return self._dehtmled_field("mobile")
    
    def clean_address(self):
        """validate address"""
        return self._dehtmled_field("address")
    
    def clean_address2(self):
        """valiadate address line 2"""
        return self._dehtmled_field("address2")
    
    def clean_address3(self):
        """validate address line 3"""
        return self._dehtmled_field("address3")
    
    def clean_message(self):
        """validate message"""
        message = self._dehtmled_field("message", allow_spaces=True)
        if len(message) > 10000:
            raise ValidationError(ugettext("Your message is too long"))
        return message
    
    def clean_groups(self):
        """validate groups"""
        try:
            groups = [Group.objects.get(id=group_id) for group_id in self.cleaned_data['groups']]
        except Group.DoesNotExist:
            raise ValidationError(ugettext("Invalid group"))
        return groups
    
    def clean_action_types(self):
        """validate action types"""
        try:
            action_types = [ActionType.objects.get(id=at_id) for at_id in self.cleaned_data['action_types']]
        except ActionType.DoesNotExist:
            raise ValidationError(ugettext("Invalid action type"))
        return action_types

    def save(self, request=None):
        """save"""
        contact = super(SubscribeForm, self).save(commit=False)
        contact.entity = self.get_entity()
        contact.city = self.cleaned_data['city']
        contact.favorite_language = self.cleaned_data.get('favorite_language', '')
        contact.save()
        # delete unknown contacts for the current entity
        contact.entity.contact_set.filter(lastname='', firstname='').exclude(id=contact.id).delete()
        
        # force also the city on the entity
        contact.entity.city = contact.city

        groups = self.cleaned_data['groups']
        for group in groups:
            contact.entity.group_set.add(group)
        contact.entity.save()
        
        subscriptions = self._save_subscription_types(contact)
        
        message = self.cleaned_data["message"]

        if message:
            action_type = ActionType.objects.get_or_create(name=ugettext("Message"))[0]
            action = Action.objects.create(
                subject=ugettext("Message from web site"),
                type=action_type,
                planned_date=datetime.now(),
                detail=message,
                display_on_board=True
            )
            action.contacts.add(contact)
            action.save()
        
        if subscriptions:
            create_subscription_action(contact, subscriptions)
        
        action_types = self.cleaned_data['action_types']
        actions = []
        for action_type in action_types:
            action = Action.objects.create(
                subject=ugettext("Contact"),
                type=action_type,
                planned_date=datetime.now(),
                display_on_board=True
            )
            action.contacts.add(contact)
            action.save()
            actions.append(action)
            
        # send an email
        send_notification_email(request, contact, actions, message)
        
        return contact


class NewsletterSchedulingForm(forms.ModelForm):
    """Define the newsletter sending date"""
    class Meta:
        model = models.Emailing
        fields = ('scheduling_dt',)

    def __init__(self, *args, **kwargs):
        kwargs['initial'] = {'scheduling_dt': dt_now() + timedelta(minutes=5)}
        super(NewsletterSchedulingForm, self).__init__(*args, **kwargs)

    def clean_scheduling_dt(self):
        """validate datetime"""
        sch_dt = self.cleaned_data['scheduling_dt']

        if not sch_dt:
            raise ValidationError(ugettext("This field is required"))

        if sch_dt < dt_now():
            raise ValidationError(ugettext("The scheduling date must be in future"))

        return sch_dt


class MinimalSubscribeForm(BetterBsModelForm, SubscriptionTypeFormMixin):
    """Subscribe to emailing"""
    subscription_required = True

    favorite_language = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = Contact
        fields = (
            'email', 'firstname', 'lastname'
        )
        widgets = {
            'lastname': forms.TextInput(attrs={'placeholder': _('Lastname')}),
            'firstname': forms.TextInput(attrs={'placeholder': _('Firstname')}),
            'email': forms.TextInput(attrs={'placeholder': _('Email'), 'required': 'required'}),
        }

    def __init__(self, *args, **kwargs):
        super(MinimalSubscribeForm, self).__init__(*args, **kwargs)

        self.fields['email'].required = True

        self._add_subscription_types_field()

        if crm_settings.has_language_choices():
            self.fields['favorite_language'].initial = get_language()

    def get_entity(self):
        """get entity from form"""
        name = '{0} {1}'.format(self.cleaned_data['lastname'], self.cleaned_data['firstname'])
        if crm_settings.ALLOW_SINGLE_CONTACT:
            return Entity.objects.create(name=name, type=None, is_single_contact=True)
        else:
            et_id = getattr(settings, 'BALAFON_INDIVIDUAL_ENTITY_ID', 1)
            entity_type = EntityType.objects.get(id=et_id)
            return Entity.objects.create(name=name, type=entity_type)

    def save(self, request=None):
        """save"""
        contact = super(MinimalSubscribeForm, self).save(commit=False)
        contact.entity = self.get_entity()
        contact.favorite_language = self.cleaned_data.get('favorite_language', '')
        contact.save()
        # delete unknown contacts for the current entity
        contact.entity.contact_set.filter(lastname='', firstname='').exclude(id=contact.id).delete()

        subscriptions = self._save_subscription_types(contact)
        if subscriptions:
            create_subscription_action(contact, subscriptions)

        # send an email
        send_notification_email(request, contact, [], '')

        return contact



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from django_extensions.db.models import TimeStampedModel

from balafon.Users.models import Favorite


@python_2_unicode_compatible
class Search(TimeStampedModel):
    """A search"""
    name = models.CharField(_('name'), max_length=100)
    
    favorites = GenericRelation(Favorite)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('search')
        verbose_name_plural = _('searchs')


@python_2_unicode_compatible
class SearchGroup(models.Model):
    """blocks"""
    search = models.ForeignKey(Search, verbose_name=_('search'))
    name = models.CharField(_('name'), max_length=100)
    
    def __str__(self):
        return '{0} {1}'.format(self.search, self.name)

    class Meta:
        verbose_name = _('search group')
        verbose_name_plural = _('search groups')


@python_2_unicode_compatible
class SearchField(models.Model):
    """fields"""
    search_group = models.ForeignKey(SearchGroup, verbose_name=_('search group'))
    field = models.CharField(_('field'), max_length=100)
    value = models.CharField(_('value'), max_length=200)
    is_list = models.BooleanField(default=False)
    count = models.IntegerField(default=0)
    
    def __str__(self):
        return '{0} {1}'.format(self.search_group, self.field)

    class Meta:
        verbose_name = _('search field')
        verbose_name_plural = _('search fied')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-09 13:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Profile', '0004_auto_20160601_1128'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contactprofile',
            name='entity_name',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='Entity name'),
        ),
        migrations.AlterField(
            model_name='contactprofile',
            name='gender',
            field=models.IntegerField(blank=True, choices=[(0, ''), (1, 'Mr'), (2, 'Mrs')], default=0, verbose_name='Gender'),
        ),
        migrations.AlterField(
            model_name='contactprofile',
            name='groups_ids',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='contactprofile',
            name='subscriptions_ids',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
    ]



# -*- coding: utf-8 -*-
"""search test package"""

from __future__ import unicode_literals

from django.contrib.auth.models import User

from coop_cms.utils import RequestManager

from balafon.unit_tests import TestCase


class BaseTestCase(TestCase):
    """Base class for search tests"""

    def setUp(self):
        """before"""
        super(BaseTestCase, self).setUp()
        self.user = User.objects.create(username="toto")
        self.user.set_password("abc")
        self.user.is_staff = True
        self.user.save()
        self._login()

    def _login(self):
        """login"""
        is_logged = self.client.login(username="toto", password="abc")
        if not is_logged:
            raise Exception("login failed")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def store_assign_analysis_code(apps, schema_editor):
    """Assign the Internet AalysisCode to all existing sales"""
    sale_analysis_code_class = apps.get_model("Store", "SaleAnalysisCode")
    sale_class = apps.get_model("Store", "Sale")

    sale_analysis_code = sale_analysis_code_class.objects.get_or_create(name=u"Internet")[0]

    for sale in sale_class.objects.filter(analysis_code__isnull=True):
        sale.analysis_code = sale_analysis_code
        sale.save()


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0002_auto_20160503_2223'),
    ]

    operations = [
        migrations.RunPython(store_assign_analysis_code),
    ]



# -*- coding: utf-8 -*-
"""test quick search"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from model_mommy import mommy

from balafon.Crm import models

from balafon.Search.tests import BaseTestCase


class QuickSearchTest(BaseTestCase):
    """Quick search"""

    def test_quick_search_city_start(self):
        """quick search on city: start with text"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCD"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        url = reverse("quick_search")
        data = {"text": "Zoo"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, city1.name)
        self.assertNotContains(response, city2.name)

    def test_quick_search_city_contains_1(self):
        """quick search on city: contains text only 1"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCD"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        url = reverse("quick_search")
        data = {"text": "oo"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, city1.name)
        self.assertNotContains(response, city2.name)

    def test_quick_search_city_contains_2(self):
        """quick search on city: contains text all"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCD"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        entity3 = mommy.make(models.Entity)
        contact3 = entity3.default_contact
        contact3.lastname = "IJKL"
        contact3.main_contact = True
        contact3.has_left = False
        contact3.city = city2
        contact3.save()

        url = reverse("quick_search")
        data = {"text": "oo"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, city1.name)
        self.assertContains(response, city2.name)

    def test_contact_by_name(self):
        """quick search by name"""
        anakin = mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker")
        obi = mommy.make(models.Contact, firstname="Obi-Wan", lastname="Kenobi")
        doe = mommy.make(models.Contact, firstname="Benjamin", lastname="Doe")
        star_wars_company = mommy.make(models.Entity)
        luke = mommy.make(models.Contact, firstname="Luuukeeee", lastname="Skywalker", entity=star_wars_company)

        url = reverse('quick_search')

        data = {"text": "Skywalker"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.firstname)
        self.assertNotContains(response, obi.firstname)
        self.assertContains(response, luke.firstname)
        self.assertNotContains(response, doe.firstname)

    def test_quick_search_anonymous(self):
        """quick search without login in"""

        mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker")

        url = reverse('quick_search')

        data = {"text": "Skywalker"}

        self.client.logout()
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)

    def test_quick_search_non_staff(self):
        """quick search without being staff member"""

        mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker")

        url = reverse('quick_search')

        data = {"text": "Skywalker"}

        self.user.is_staff = False
        self.user.save()

        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)

    def test_has_left_contact_by_name(self):
        """quick search by name for contact who left"""
        start_wars_company = mommy.make(models.Entity)
        luke = mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=start_wars_company)
        anakin = mommy.make(
            models.Contact, firstname="Anakin", lastname="Skywalker", entity=start_wars_company, has_left=True
        )

        url = reverse('quick_search')

        data = {"text": "Skywalker"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.firstname)
        self.assertContains(response, luke.firstname)

    def test_has_left_contact_by_entity_name(self):
        """quick search by company name with contacts who left"""
        star_wars_company = mommy.make(models.Entity, name="Star-Wars", is_single_contact=False)
        luke = mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company)
        anakin = mommy.make(
            models.Contact, firstname="Anakin", lastname="Skywalker", entity=star_wars_company, has_left=True
        )
        leia = mommy.make(models.Contact, firstname="Leia", lastname="Skywalker", entity=star_wars_company)
        shmi = mommy.make(
            models.Contact, firstname="Shmi", lastname="Skywalker", entity=star_wars_company, main_contact=False
        )

        url = reverse('quick_search')

        data = {"text": "Star-Wars"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, anakin.firstname)
        self.assertNotContains(response, shmi.firstname)
        self.assertContains(response, luke.firstname)
        self.assertContains(response, leia.firstname)

    def test_contact_by_entity_name(self):
        """quick search by entity name"""
        doe = mommy.make(models.Contact, firstname="Doe", lastname="John")
        star_wars_company = mommy.make(models.Entity, name="Starwars")
        luke = mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company)
        star_trek_company = mommy.make(models.Entity, name="StarTrek")
        piccard = mommy.make(models.Contact, firstname="Jean-Luc", lastname="Piccard", entity=star_trek_company)
        dark_side_company = mommy.make(models.Entity, name="DarkSide")
        palpatine = mommy.make(models.Contact, firstname="Senateur", lastname="Palpatine", entity=dark_side_company)

        url = reverse('quick_search')

        data = {"text": "Star"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, piccard.firstname)
        self.assertContains(response, luke.firstname)
        self.assertNotContains(response, doe.firstname)
        self.assertNotContains(response, palpatine.firstname)

    def test_contact_by_phone(self):
        """quick search by phone"""
        anakin = mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker", phone="04.99.99.99.99")
        obi = mommy.make(models.Contact, firstname="Obi-Wan", lastname="Kenobi", mobile="04.99.00.00.00")
        doe = mommy.make(models.Contact, firstname="Doe", lastname="John", phone="03.33.33.33.33")
        star_wars_company = mommy.make(models.Entity, phone="04.99.11.22.33")
        mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company)

        url = reverse('quick_search')

        data = {"text": "04.99"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.phone)
        self.assertContains(response, obi.mobile)
        self.assertContains(response, star_wars_company.phone)
        self.assertNotContains(response, doe.phone)

        self.assertContains(response, anakin.firstname)
        self.assertContains(response, obi.firstname)
        self.assertContains(response, star_wars_company.name)
        self.assertNotContains(response, doe.firstname)

    def test_contact_by_phone_has_left(self):
        """quick search by phone with contacts who left"""
        anakin = mommy.make(
            models.Contact, firstname="Anakin", lastname="Skywalker", phone="04.99.99.99.99", has_left=True
        )
        obi = mommy.make(
            models.Contact, firstname="Obi-Wan", lastname="Kenobi", mobile="04.99.00.00.00", has_left=True
        )
        doe = mommy.make(models.Contact, firstname="DoeDoeDoe", lastname="JohnJohnJohn", phone="03.33.33.33.33")
        star_wars_company = mommy.make(models.Entity, phone="04.99.11.22.33")
        mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company, has_left=True)

        url = reverse('quick_search')

        data = {"text": "04.99"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.phone)
        self.assertContains(response, obi.mobile)
        self.assertContains(response, star_wars_company.phone)
        self.assertNotContains(response, doe.phone)

        self.assertContains(response, anakin.firstname)
        self.assertContains(response, obi.firstname)
        self.assertContains(response, star_wars_company.name)
        self.assertNotContains(response, doe.firstname)

    def test_contact_by_email(self):
        """quick search by email"""
        anakin = mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker", email="anakin@starwars.com")
        obi = mommy.make(models.Contact, firstname="Obi-Wan", lastname="Kenobi", email="obiwan@starwars.com")
        doe = mommy.make(models.Contact, firstname="Doe", lastname="John", email="john.doe@toto.fr")
        star_wars_company = mommy.make(models.Entity, email="contact@starwars.com")
        luke = mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company)
        sidious = mommy.make(
            models.Contact, firstname="Dark", lastname="Sidious", entity=star_wars_company, email="sidious@darkside.com"
        )

        url = reverse('quick_search')

        data = {"text": "@starwars.com"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.email)
        self.assertContains(response, obi.email)
        self.assertContains(response, star_wars_company.email)
        self.assertNotContains(response, doe.email)
        self.assertNotContains(response, sidious.email)

        self.assertContains(response, anakin.firstname)
        self.assertContains(response, obi.firstname)
        self.assertContains(response, luke.firstname)
        self.assertNotContains(response, doe.firstname)
        self.assertNotContains(response, sidious.firstname)

    def test_contact_by_email_has_left(self):
        """quick search by contacts who left"""
        anakin = mommy.make(
            models.Contact, firstname="Anakin", lastname="Skywalker", email="anakin@starwars.com", has_left=True
        )
        obi = mommy.make(
            models.Contact, firstname="Obi-Wan", lastname="Kenobi", email="obiwan@starwars.com", has_left=True
        )
        doe = mommy.make(models.Contact, firstname="Doe", lastname="John", email="john.doe@toto.fr")
        star_wars_company = mommy.make(models.Entity, email="contact@starwars.com")
        luke = mommy.make(
            models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company, has_left=True
        )
        sidious = mommy.make(
            models.Contact, firstname="Dark", lastname="Sidious", entity=star_wars_company, email="sidious@darkside.com"
        )

        url = reverse('quick_search')

        data = {"text": "@starwars.com"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.email)
        self.assertContains(response, obi.email)
        self.assertContains(response, star_wars_company.email)
        self.assertNotContains(response, doe.email)
        self.assertNotContains(response, sidious.email)

        self.assertContains(response, anakin.firstname)
        self.assertContains(response, obi.firstname)
        self.assertContains(response, luke.firstname)
        self.assertNotContains(response, doe.firstname)
        self.assertNotContains(response, sidious.firstname)

    def test_contact_by_email_no_at(self):
        """quick search by email: no @ in text"""
        anakin = mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker", email="anakin@starwars.com")
        obi = mommy.make(models.Contact, firstname="Obi-Wan", lastname="Kenobi", email="obiwan@starwars.com")
        doe = mommy.make(models.Contact, firstname="Doe", lastname="John", email="john.doe@toto.fr")
        star_wars_company = mommy.make(models.Entity, email="contact@starwars.com")
        luke = mommy.make(models.Contact, firstname="Luke", lastname="Skywalker", entity=star_wars_company)
        sidious = mommy.make(
            models.Contact, firstname="Dark", lastname="Sidious", entity=star_wars_company, email="sidious@darkside.com"
        )

        url = reverse('quick_search')

        data = {"text": "starwars.com"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, anakin.email)
        self.assertContains(response, obi.email)
        self.assertContains(response, star_wars_company.email)
        self.assertNotContains(response, doe.email)
        self.assertNotContains(response, sidious.email)

        self.assertContains(response, anakin.firstname)
        self.assertContains(response, obi.firstname)
        self.assertContains(response, luke.firstname)
        self.assertNotContains(response, doe.firstname)
        self.assertNotContains(response, sidious.firstname)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.admin.widgets import ManyToManyRawIdWidget
from django.core.urlresolvers import reverse
from django.utils.encoding import smart_text
from django.utils.html import escape

import floppyforms.__future__ as forms


class VerboseManyToManyRawIdWidget(ManyToManyRawIdWidget):
    """
    A Widget for displaying ManyToMany ids in the "raw_id" interface rather than
    in a <select multiple> box. Display user-friendly value like the ForeignKeyRawId widget
    """
    def __init__(self, rel, attrs=None, *args, **kwargs):
        super(VerboseManyToManyRawIdWidget, self).__init__(rel, attrs, *args, **kwargs)
        
    def label_for_value(self, value):
        values = value.split(',')
        str_values = []
        key = self.rel.get_related_field().name
        app_label = self.rel.to._meta.app_label
        class_name = self.rel.to._meta.object_name.lower()
        for v in values:
            try:
                obj = self.rel.to._default_manager.using(self.db).get(**{key: v})
                url = reverse('admin:{0}_{1}_change'.format(app_label, class_name), args=[obj.id])
                label = escape(smart_text(obj))
                
                x = '<a href="{0}" {1}>{2}</a>'.format(
                    url,
                    'onclick="return showAddAnotherPopup(this);" target="_blank"',
                    label
                )
                
                str_values += [x]
            except self.rel.to.DoesNotExist:
                str_values += ['???']
        return '&nbsp;<strong>{0}</strong>'.format(',&nbsp;'.join(str_values))


class CalcHiddenInput(forms.HiddenInput):
    """This field is calculated and hidden. It is not in POST data"""

    def value_omitted_from_data(self, data, files, name):
        # Tell django that this is field value doesn't come from POST data
        return False



# -*- coding: utf-8 -*-
"""unit testing"""

# pylint: disable=too-many-locals
# pylint: disable=too-many-statements
# pylint: disable=too-many-branches

from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.core.urlresolvers import reverse

from model_mommy import mommy
from rest_framework import status
from rest_framework.test import APITestCase, APIClient

from balafon.Crm import models


class AboutMeTest(APITestCase):
    """It should return actions"""

    def setUp(self):
        self.client = APIClient()

    def test_get_about_me_staff(self):
        """It should return info about user"""
        user = mommy.make(User, username='joe', is_active=True, is_staff=True)
        user.set_password('pass')
        user.save()

        self.client.login(username=user.username, password='pass')

        url = reverse('crm_api_about_me')

        response = self.client.get(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(1, len(response.data))
        self.assertEqual(user.last_name, response.data[0]['last_name'])
        self.assertEqual(user.first_name, response.data[0]['first_name'])
        self.assertEqual(None, response.data[0].get('password'))

    def test_get_about_me_team_member(self):
        """It should return info about user"""
        user = mommy.make(User, username='joe', is_active=True, is_staff=False)
        user.set_password('pass')
        user.save()

        mommy.make(models.TeamMember, user=user)

        self.client.login(username=user.username, password='pass')

        url = reverse('crm_api_about_me')

        response = self.client.get(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(1, len(response.data))
        self.assertEqual(user.last_name, response.data[0]['last_name'])
        self.assertEqual(user.first_name, response.data[0]['first_name'])
        self.assertEqual(None, response.data[0].get('password'))

    def test_get_about_me_anynone(self):
        """It should return info about user"""
        user = mommy.make(User, username='joe', is_active=True, is_staff=False)
        user.set_password('pass')
        user.save()

        self.client.login(username=user.username, password='pass')

        url = reverse('crm_api_about_me')

        response = self.client.get(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(1, len(response.data))
        self.assertEqual(user.last_name, response.data[0]['last_name'])
        self.assertEqual(user.first_name, response.data[0]['first_name'])
        self.assertEqual(None, response.data[0].get('password'))

    def test_get_about_me_anonymous(self):
        """It should return error"""
        url = reverse('crm_api_about_me')

        response = self.client.get(url, format='json')
        self.assertTrue(response.status_code in (status.HTTP_401_UNAUTHORIZED, status.HTTP_403_FORBIDDEN))

    def test_get_about_me_inactive(self):
        """It should return info about user"""
        user = mommy.make(User, username='joe', is_active=True, is_staff=False)
        user.set_password('pass')
        user.save()

        self.client.login(username=user.username, password='pass')

        user.is_active = False
        user.save()

        url = reverse('crm_api_about_me')

        response = self.client.get(url, format='json')
        self.assertTrue(response.status_code in (status.HTTP_401_UNAUTHORIZED, status.HTTP_403_FORBIDDEN))


class ContactOrEntityTest(APITestCase):
    """test search contacts or entities"""

    def _login(self):
        user = mommy.make(User, username='joe', is_active=True, is_staff=True)
        user.set_password('pass')
        user.save()

        self.client.login(username=user.username, password='pass')

        return user

    def test_api_not_logged(self):
        """can't access"""
        url = reverse('crm_api_contacts_or_entities') + "?name=A"
        response = self.client.get(url)
        self.assertTrue(response.status_code in (status.HTTP_401_UNAUTHORIZED, status.HTTP_403_FORBIDDEN))

    def test_api_not_admin(self):
        """can't access"""
        user = self._login()
        user.is_staff = False
        user.save()
        url = reverse('crm_api_contacts_or_entities') + "?name=A"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_api_empty(self):
        """returns empty list"""
        self._login()
        url = reverse('crm_api_contacts_or_entities') + "?name=A"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, [])

    def test_api_both(self):
        """returns list of contacts and entities"""

        contact_type = ContentType.objects.get_for_model(models.Contact)
        entity_type = ContentType.objects.get_for_model(models.Entity)

        entity1 = mommy.make(models.Entity, name='Alpha', is_single_contact=False)
        entity2 = mommy.make(models.Entity, name='Beta', is_single_contact=False)

        entity3 = mommy.make(models.Entity, name='ALLARD', is_single_contact=True)
        contact1 = mommy.make(models.Contact, lastname='Allard', entity=entity3)

        entity4 = mommy.make(models.Entity, name='Zardo', is_single_contact=False)
        contact2 = mommy.make(models.Contact, lastname='Bernard')

        self._login()

        url = reverse('crm_api_contacts_or_entities') + "?name=A"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, [
            {'id': contact1.id, 'name': 'Allard', 'type': contact_type.id, 'city': ''},
            {'id': entity1.id, 'name': 'Alpha', 'type': entity_type.id, 'city': ''},
        ])

    def test_api_cities(self):
        """returns list of contacts and entities"""

        contact_type = ContentType.objects.get_for_model(models.Contact)
        entity_type = ContentType.objects.get_for_model(models.Entity)

        default_country = mommy.make(models.Zone, name=settings.BALAFON_DEFAULT_COUNTRY, parent=None)
        foreign_country = mommy.make(models.Zone, name="BB", parent=None)
        parent = mommy.make(models.Zone, name="AA", code="42", parent=default_country)

        city1 = mommy.make(models.City, name="abcd", parent=parent)
        city2 = mommy.make(models.City, name="efgh", parent=parent)
        city3 = mommy.make(models.City, name="ijkl", parent=foreign_country)

        entity1 = mommy.make(models.Entity, name='Alpha', is_single_contact=False, city=city1)

        entity2 = mommy.make(models.Entity, name='ENTITY2', is_single_contact=False, city=city2)
        contact2 = mommy.make(models.Contact, lastname='Allard', entity=entity2)

        entity3 = mommy.make(models.Entity, name='ENTITY3', is_single_contact=False, city=city2)
        contact3 = mommy.make(models.Contact, lastname='Azerty', entity=entity3, city=city3)

        self._login()

        url = reverse('crm_api_contacts_or_entities') + "?name=A"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, [
            {'id': contact2.id,'name': 'Allard', 'type': contact_type.id, 'city': city2.get_friendly_name()},
            {'id': entity1.id, 'name': 'Alpha', 'type': entity_type.id, 'city': city1.get_friendly_name()},
            {'id': contact3.id, 'name': 'Azerty', 'type': contact_type.id, 'city': city3.get_friendly_name()},
        ])

    def test_api_limit(self):
        """returns list of contacts and entities"""
        for i in range(20):
            mommy.make(models.Entity, name='Alpha', is_single_contact=False)
            mommy.make(models.Entity, name='Beta', is_single_contact=False)

        entity = mommy.make(models.Entity, name='ZZ', is_single_contact=True)
        for i in range(20):
            mommy.make(models.Contact, lastname='Allard', entity=entity)
            mommy.make(models.Contact, lastname='Bernard', entity=entity)

        self._login()

        url = reverse('crm_api_contacts_or_entities') + "?name=A"
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 10)
        for elt in response.data:
            self.assertEqual(elt['name'][0], 'A')



# -*- coding: utf-8 -*-
"""test email sending"""

from __future__ import unicode_literals

from datetime import datetime
from unittest import skipIf

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import management
from django.core import mail
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from django.utils.translation import activate

from coop_cms.models import Newsletter
from coop_cms.settings import is_localized, is_multilang
from coop_cms.tests.test_newsletter import NewsletterTest as BaseNewsletterTest
from coop_cms.utils import get_url_in_language
from model_mommy import mommy

from balafon.Crm import models
from balafon.Emailing.models import Emailing, MagicLink
from balafon.Emailing.tests import BaseTestCase
from balafon.Emailing.utils import force_line_max_length


class SendEmailingTest(BaseTestCase):

    def setUp(self):
        activate(settings.LANGUAGES[0][0])
        super(SendEmailingTest, self).setUp()
        settings.COOP_CMS_FROM_EMAIL = 'toto@toto.fr'
        settings.COOP_CMS_REPLY_TO = 'titi@toto.fr'

        site = Site.objects.get_current()
        site.domain = settings.COOP_CMS_SITE_PREFIX
        site.save()

    def tearDown(self):
        activate(settings.LANGUAGES[0][0])

    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter(self):

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(
                models.Contact, entity=entity,
                email=name+'@toto.fr', lastname=name.capitalize()
            ) for name in names
        ]

        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site)
        )

        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        # check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda e: e.to)
        contacts.sort(key=lambda c: c.get_email)

        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_unregister', args=[emailing.id, contact.uuid]
            )

            self.assertEqual(email.to, [contact.get_email_address()])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find(entity.name) >= 0)
            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
            )

            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")

            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)

            # check magic links
            self.assertTrue(MagicLink.objects.count() > 0)

            # check an action has been created
            c = models.Contact.objects.get(id=contact.id)
            self.assertEqual(c.action_set.count(), 1)
            self.assertEqual(c.action_set.all()[0].subject, email.subject)

    @skipIf(len(settings.LANGUAGES) < 2, "LANGUAGES less than 2")
    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_language(self):

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(
                models.Contact, entity=entity,
                email=name+'@toto.fr',
                lastname=name.capitalize(),
                firstname=name,
            ) for name in names
        ]

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]

        content = '<h2>Hello #!-fullname-!#!</h2><p>{0}Visit <a href="http://toto.{0}">{0}</a>'
        content += '<a href="mailto:me@me.{0}">mailme</a><a href="#art1">internal link</a></p>'

        newsletter_data = {
            'subject_'+origin_lang: 'This is the {0} subject'.format(origin_lang),
            'subject_'+trans_lang: 'This is the {0} subject'.format(trans_lang),
            'content_'+origin_lang: content.format(origin_lang, ),
            'content_'+trans_lang: content.format(trans_lang),
            'template': 'test/newsletter_contact_lang.html'
        }

        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site),
            lang=trans_lang
        )

        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        #check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda e: e.to)
        contacts.sort(key=lambda c: c.get_email)

        activate(trans_lang)
        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_unregister', args=[emailing.id, contact.uuid]
            )

            view_en_url = reverse("emailing_view_online_lang", args=[emailing.id, contact.uuid, 'en'])

            self.assertEqual(email.to, [contact.get_email_address()])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject_'+trans_lang])
            self.assertTrue(email.body.find(entity.name) >= 0)
            #print email.body
            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")

            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
            #Check links are not magic
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(view_en_url) >= 0)
            #Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.{0}".format(trans_lang)) > 0)
            #Check internal links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)

            #check magic links
            self.assertTrue(MagicLink.objects.count() > 0)

            #check an action has been created
            c = models.Contact.objects.get(id=contact.id)
            self.assertEqual(c.action_set.count(), 1)
            self.assertEqual(c.action_set.all()[0].subject, email.subject)

    @skipIf(len(settings.LANGUAGES) < 2, "LANGUAGES less than 2")
    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_contact_language(self):
        """test that we use the favorite language of the contact when sending him a newsletter"""

        entity = mommy.make(models.Entity, name="my corp")

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]

        names = ['alpha', 'beta', 'gamma']
        langs = ['', origin_lang, trans_lang]

        contacts = [
            mommy.make(
                models.Contact,
                entity=entity,
                email=name+'@toto.fr',
                lastname=name.capitalize(),
                firstname=name,
                favorite_language=lang
            ) for (name, lang) in zip(names, langs)
        ]

        content = '<h2>Hello #!-fullname-!#!</h2><p>{0}Visit <a href="http://toto.{0}">{0}</a>'
        content += '<a href="mailto:me@me.{0}">mailme</a><a href="#art1">internal link</a></p>'

        newsletter_data = {
            'subject_' + origin_lang: 'This is the {0} subject'.format(origin_lang),
            'subject_' + trans_lang: 'This is the {0} subject'.format(trans_lang),
            'content_' + origin_lang: content.format(origin_lang),
            'content_' + trans_lang: content.format(trans_lang),
            'template': 'test/newsletter_contact.html'
        }

        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site),
            lang=''
        )

        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        activate(origin_lang)

        # check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda email: email.to)
        contacts.sort(key=lambda contact: contact.get_email)

        activate(trans_lang)
        for email, contact, lang in zip(outbox, contacts, langs):
            viewonline_url = reverse(
                'emailing_view_online', args=[emailing.id, contact.uuid]
            )

            unsubscribe_url = reverse(
                'emailing_unregister', args=[emailing.id, contact.uuid]
            )

            contact_lang = lang or origin_lang
            viewonline_url = get_url_in_language(viewonline_url, contact_lang)
            unsubscribe_url = get_url_in_language(unsubscribe_url, contact_lang)

            viewonline_url = emailing.get_domain_url_prefix() + viewonline_url
            unsubscribe_url = emailing.get_domain_url_prefix() + unsubscribe_url

            self.assertEqual(email.to, [contact.get_email_address()])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)

            self.assertEqual(email.subject, newsletter_data['subject_'+contact_lang])
            self.assertTrue(email.body.find(entity.name) >= 0)
            # print email.body
            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
            )

            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")

            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.{0}".format(contact_lang)) > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)

            # check magic links
            self.assertTrue(MagicLink.objects.count() > 0)

            # check an action has been created
            contact = models.Contact.objects.get(id=contact.id)
            self.assertEqual(contact.action_set.count(), 1)
            self.assertEqual(contact.action_set.all()[0].subject, email.subject)

    @override_settings(COOP_CMS_REPLY_TO="reply_to@toto.fr")
    def test_send_newsletter_reply_to(self):

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(
                models.Contact, entity=entity,
                email=name+'@toto.fr', lastname=name.capitalize()
            ) for name in names
        ]

        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site)
        )

        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        # check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda e: e.to)
        contacts.sort(key=lambda c: c.get_email)

        for email, contact in zip(outbox, contacts):
            viewonline_url = reverse(
                'emailing_view_online', args=[emailing.id, contact.uuid]
            )

            unsubscribe_url = reverse(
                'emailing_unregister', args=[emailing.id, contact.uuid]
            )

            viewonline_url = emailing.get_domain_url_prefix() + viewonline_url
            unsubscribe_url = emailing.get_domain_url_prefix() + unsubscribe_url

            self.assertEqual(email.to, [contact.get_email_address()])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find(entity.name) >= 0)
            #print email.body
            self.assertEqual(email.extra_headers['Reply-To'], settings.COOP_CMS_REPLY_TO)
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, settings.COOP_CMS_REPLY_TO)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)

            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)

            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)

            # check magic links
            self.assertTrue(MagicLink.objects.count() > 0)

            # check an action has been created
            c = models.Contact.objects.get(id=contact.id)
            self.assertEqual(c.action_set.count(), 1)
            self.assertEqual(c.action_set.all()[0].subject, email.subject)

    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_from_email(self):

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(
                models.Contact, entity=entity,
                email=name+'@toto.fr', lastname=name.capitalize()
            ) for name in names
        ]

        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site),
            from_email="abcd@defg.fr"
        )

        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        #check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda e: e.to)
        contacts.sort(key=lambda c: c.get_email)

        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_unregister', args=[emailing.id, contact.uuid]
            )

            self.assertEqual(email.to, [contact.get_email_address()])
            self.assertEqual(email.from_email, emailing.from_email)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find(entity.name) >= 0)
            #print email.body
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, emailing.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)

            #Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)

            #Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)

            #check magic links
            self.assertTrue(MagicLink.objects.count() > 0)

            #check an action has been created
            c = models.Contact.objects.get(id=contact.id)
            self.assertEqual(c.action_set.count(), 1)
            self.assertEqual(c.action_set.all()[0].subject, email.subject)

    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_check_unregister_name(self):

        entity = mommy.make(models.Entity, name="my corp")

        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(
                models.Contact, entity=entity,
                email=name + '@toto.fr', lastname=name.capitalize()
            ) for name in names
            ]

        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'

        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        subscription_1 = mommy.make(models.SubscriptionType, site=site, name="MY_COMPANY_#1")
        subscription_2 = mommy.make(models.SubscriptionType, site=site, name="MY_COMPANY_#2")

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=subscription_2,
            from_email="abcd@defg.fr"
        )

        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        # check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))

        self.assertEqual(len(mail.outbox), len(contacts))

        outbox = list(mail.outbox)
        outbox.sort(key=lambda e: e.to)
        contacts.sort(key=lambda c: c.get_email)

        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'emailing_unregister', args=[emailing.id, contact.uuid]
            )

            self.assertFalse(email.body.find(subscription_1.name) >= 0)
            self.assertTrue(email.body.find(subscription_2.name) >= 0)

            self.assertEqual(email.to, [contact.get_email_address()])
            self.assertEqual(email.from_email, emailing.from_email)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find(entity.name) >= 0)
            # print email.body
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, emailing.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(entity.name) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)

            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)

            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)

            # check magic links
            self.assertTrue(MagicLink.objects.count() > 0)

            # check an action has been created
            c = models.Contact.objects.get(id=contact.id)
            self.assertEqual(c.action_set.count(), 1)
            self.assertEqual(c.action_set.all()[0].subject, email.subject)

    def test_view_magic_link(self):
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity, email='toto@toto.fr', lastname='Toto')
        emailing = mommy.make(Emailing)
        emailing.sent_to.add(contact)
        emailing.save()

        link = "http://www.google.fr"
        magic_link = MagicLink.objects.create(emailing=emailing, url=link)

        response = self.client.get(reverse('emailing_view_link', args=[magic_link.uuid, contact.uuid]))
        self.assertEqual(302, response.status_code)
        self.assertEqual(response['Location'], link)

    def test_unregister_mailinglist(self):
        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)
        third_party_subscription = mommy.make(models.SubscriptionType, name="3rd_party", site=site1)

        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Toto'
        )

        subscription1 = mommy.make(
            models.Subscription,
            subscription_type=newsletter_subscription,
            contact=contact,
            accept_subscription=True
        )

        subscription2 = mommy.make(
            models.Subscription,
            subscription_type=third_party_subscription,
            contact=contact,
            accept_subscription=True
        )

        emailing = mommy.make(Emailing, subscription_type=newsletter_subscription)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_unregister', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        response = self.client.post(url, data={'unregister': True})
        self.assertEqual(200, response.status_code)

        subscription1 = models.Subscription.objects.get(id=subscription1.id)
        self.assertEqual(subscription1.accept_subscription, False)
        self.assertEqual(subscription1.contact, contact)

        subscription2 = models.Subscription.objects.get(id=subscription2.id)
        self.assertEqual(subscription2.accept_subscription, True)
        self.assertEqual(subscription2.contact, contact)

        self.assertEqual(emailing.unsub.count(), 1)
        self.assertEqual(list(emailing.unsub.all()), [contact])

    def test_unregister_mailinglist_dont_exist(self):
        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)

        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Toto'
        )

        emailing = mommy.make(Emailing, subscription_type=newsletter_subscription)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_unregister', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        response = self.client.post(url, data={'unregister': True})
        self.assertEqual(200, response.status_code)

        subscription1 = models.Subscription.objects.get(subscription_type=newsletter_subscription)
        self.assertEqual(subscription1.accept_subscription, False)
        self.assertEqual(subscription1.contact, contact)
        self.assertEqual(emailing.unsub.count(), 1)
        self.assertEqual(list(emailing.unsub.all()), [contact])

    def test_unregister_mailinglist_twice(self):
        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)

        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Toto'
        )

        subscription1 = mommy.make(
            models.Subscription,
            subscription_type=newsletter_subscription,
            contact=contact,
            accept_subscription=False
        )

        emailing = mommy.make(Emailing, subscription_type=newsletter_subscription)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_unregister', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        response = self.client.post(url, data={'unregister': True})
        self.assertEqual(200, response.status_code)

        subscription1 = models.Subscription.objects.get(id=subscription1.id)
        self.assertEqual(subscription1.accept_subscription, False)
        self.assertEqual(subscription1.contact, contact)

        self.assertEqual(emailing.unsub.count(), 1)
        self.assertEqual(list(emailing.unsub.all()), [contact])

    def test_unregister_mailinglist_notfound_emailing(self):
        site1 = Site.objects.get_current()

        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Toto'
        )

        url = reverse('emailing_unregister', args=[1, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        response = self.client.post(url, data={'unregister': True})
        self.assertEqual(200, response.status_code)

        self.assertEqual(Emailing.objects.count(), 0)

    def test_unregister_mailinglist_not_found_contact(self):
        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)

        emailing = mommy.make(Emailing, subscription_type=newsletter_subscription)

        url = reverse('emailing_unregister', args=[emailing.id, "aaaa"])
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)

        response = self.client.post(url, data={'unregister': True})
        self.assertEqual(404, response.status_code)

        self.assertEqual(emailing.unsub.count(), 0)

    def test_view_online(self):
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Azerty', firstname='Albert'
        )
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact.fullname)
        self.assertEqual(MagicLink.objects.count(), 2)

        magic_link0 = MagicLink.objects.all()[0]
        self.assertContains(response, reverse('emailing_view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, '/this-link-without-prefix-in-template')

        magic_link1 = MagicLink.objects.all()[1]
        self.assertContains(response, reverse('emailing_view_link', args=[magic_link1.uuid, contact.uuid]))
        self.assertEqual(magic_link1.url, 'http://toto.fr')

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_emailing_view_online_lang(self):
        """test view emailing in other lang"""
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Azerty', firstname='Albert'
        )
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()

        #default_lang = settings.LANGUAGES[0][0]
        other_lang = settings.LANGUAGES[1][0]

        url = reverse('emailing_view_online_lang', args=[emailing.id, contact.uuid, other_lang])
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        next_url = reverse('emailing_view_online', args=[emailing.id, contact.uuid])[3:]
        redirect_url = other_lang + next_url
        self.assertTrue(response['Location'].find(redirect_url) >= 0)

    @override_settings(SECRET_KEY="super-héros")
    def test_view_online_utf_links(self):
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Azerty', firstname='Albert'
        )
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr/à-bientôt">à bientôt</a></p>',
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact.fullname)
        self.assertEqual(MagicLink.objects.count(), 2)

        magic_link0 = MagicLink.objects.all()[0]
        self.assertContains(response, reverse('emailing_view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, '/this-link-without-prefix-in-template')

        magic_link1 = MagicLink.objects.all()[1]
        self.assertContains(response, reverse('emailing_view_link', args=[magic_link1.uuid, contact.uuid]))
        self.assertEqual(magic_link1.url, 'http://toto.fr/à-bientôt')

    def test_view_long_links(self):
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Azerty', firstname='Albert'
        )
        short_link = "http://toto.fr/{0}".format("abcde" * 100)[:499]
        long_link = "http://toto.fr/{0}".format("abcde" * 100)  # >500 chars
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<p>Visit <a href="{0}">long link</a> <a href="{1}">long link</a></p>'.format(
                short_link, long_link),
            'template': 'test/newsletter_no_link.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        self.assertEqual(MagicLink.objects.count(), 1)

        magic_link0 = MagicLink.objects.all()[0]
        self.assertContains(response, reverse('emailing_view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, short_link)

        self.assertContains(response, long_link)

    def test_view_duplicate_links(self):
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Azerty', firstname='Albert'
        )
        short_link = "http://toto.fr/abcde/"
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<p>Visit <a href="{0}">link1</a> <a href="{1}">link2</a></p>'.format(
                short_link, short_link
            ),
            'template': 'test/newsletter_no_link.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()

        url = reverse('emailing_view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        self.assertEqual(MagicLink.objects.count(), 1)

        magic_link0 = MagicLink.objects.all()[0]
        self.assertContains(response, reverse('emailing_view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, short_link)

    def test_view_duplicate_emailing(self):
        entity = mommy.make(models.Entity, name="my corp")
        contact = mommy.make(
            models.Contact, entity=entity,
            email='toto@toto.fr', lastname='Azerty', firstname='Albert'
        )
        short_link = "http://toto.fr/abcde/"
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<p>Visit <a href="{0}">link1</a></p>'.format(
                short_link
            ),
            'template': 'test/newsletter_no_link.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing1 = mommy.make(Emailing, newsletter=newsletter)
        emailing1.sent_to.add(contact)
        emailing1.save()

        emailing2 = mommy.make(Emailing, newsletter=newsletter)
        emailing2.sent_to.add(contact)
        emailing2.save()

        for emailing in (emailing1, emailing2):
            url = reverse('emailing_view_online', args=[emailing.id, contact.uuid])
            response = self.client.get(url)
            self.assertEqual(200, response.status_code)

            magic_links = MagicLink.objects.filter(emailing=emailing)
            self.assertEqual(magic_links.count(), 1)

            magic_link0 = magic_links[0]
            self.assertContains(response, reverse('emailing_view_link', args=[magic_link0.uuid, contact.uuid]))
            self.assertEqual(magic_link0.url, short_link)


class NewsletterTest(BaseNewsletterTest):
    """test coop_cms newsletter"""

    def test_send_newsletter_template(self):
        def extra_checker(e):
            site = Site.objects.get(id=settings.SITE_ID)
            url = "http://"+site.domain+"/this-link-without-prefix-in-template"
            self.assertTrue(e.alternatives[0][0].find(url) >= 0)
        super(NewsletterTest, self).test_send_test_newsletter('test/newsletter_contact.html')


class CustomTemplateTest(BaseTestCase):
    """Test the user-templating"""

    def test_send_newsletter(self):
        entity = mommy.make(models.Entity, name="my corp")
        city = mommy.make(models.City, name='MaVille')

        contact = mommy.make(
            models.Contact,
            gender=models.Contact.GENDER_MALE,
            entity=entity,
            email='john@toto.fr',
            lastname='Doe',
            firstname='John',
            address='2 rue de la lune',
            zip_code='42424',
            city=city
        )

        content = '''
        <h2>Hello #!-gender_dear-!# #!-gender_name-!# #!-firstname-!# #!-lastname-!#</h2>
        Email: #!-email-!#
        Addresse: #!-address-!# #!-zip_code-!# #!-city_name-!#
        Address: #!-full_address-!#
        Entity: #!-entity_name-!#
        '''

        newsletter_data = {
            'subject': 'Hello #!-firstname-!#',
            'content': content,
            'template': 'test/newsletter_contact.html',
        }

        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            Emailing,
            newsletter=newsletter,
            status=Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site),
        )

        emailing.send_to.add(contact)
        emailing.save()

        management.call_command('emailing_scheduler', verbosity=0, interactive=False)

        emailing = Emailing.objects.get(id=emailing.id)

        # check emailing status
        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), 1)

        self.assertEqual(len(mail.outbox), 1)

        email = mail.outbox[0]

        self.assertEqual(email.to, [contact.get_email_address()])
        self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)

        self.assertTrue(email.alternatives[0][1], "text/html")

        html_body = email.alternatives[0][0]
        self.assertTrue(html_body.find(contact.fullname) >= 0)

        self.assertEqual(email.subject, 'Hello John')

        fields = (
            'firstname', 'lastname', 'gender_dear', 'gender_name', 'email', 'city_name', 'address', 'zip_code',
            'full_address', 'entity_name'
        )
        for field_name in fields:
            self.assertTrue(email.body.find(getattr(contact, field_name)) >= 0)
            self.assertTrue(html_body.find(getattr(contact, field_name)) >= 0)


class MaxLineLengthTest(BaseTestCase):
    """Check that we can control the max length of line"""

    def test_force_line_max_length(self):
        """it should not have a line more than 400 chars"""

        in_lines = [
            'This is a messsage',
            "123456789 " * 50,
            "That's all folks!"
        ]
        text = "\n".join(in_lines)

        out_text = force_line_max_length(text, max_length_per_line=400, dont_cut_in_quotes=True)
        out_lines = out_text.split("\n")

        self.assertEqual(len(out_lines), 4)
        self.assertEqual(out_lines[0], in_lines[0])
        self.assertEqual(out_lines[1], "123456789 " * 40 + "123456789")
        self.assertEqual(out_lines[2], "123456789 " * 8 + "123456789")
        self.assertEqual(out_lines[3], in_lines[2])

    def test_force_line_max_length_in_p(self):
        """it should not have a line more than 400 chars"""

        content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>
            <p>{0}</p>
            </html>
        """.format("123456789 " * 50)

        expected_content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>\n<p>{0}\n{1}</p>
            </html>
        """.format(
            "123456789 " * 39 + "123456789",
            "123456789 " * 10
        )

        result = force_line_max_length(content, max_length_per_line=400, dont_cut_in_quotes=False)
        self.assertEqual(expected_content, result)

    def test_force_line_max_length_a1(self):
        """it should not have a line more than 400 chars"""

        content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>
            <p>{0}<a href="{1}">This is a link</a></p>
            </html>
        """.format("123456789 " * 36, "this-is-a-link-dont-cut-in-quotes")

        expected_content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>\n<p>{0}<a href="{1}">This\nis a link</a></p>
            </html>
        """.format("123456789 " * 36, "this-is-a-link-dont-cut-in-quotes")

        result = force_line_max_length(content,  max_length_per_line=400, dont_cut_in_quotes=False)

        self.assertEqual(expected_content, result)

    def test_force_line_max_length_a2(self):
        """it should not have a line more than 400 chars"""

        content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>
            <p>{0}<a class="abcdefgh ijklmnop qrstuvwxyz" href="{1}">This is a link</a></p>
            </html>
        """.format("123456789 " * 38, "this-is-a-link-dont-cut-in-quotes")

        expected_content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>\n<p>{0}<a class="abcdefgh ijklmnop qrstuvwxyz"\nhref="{1}">This is a link</a></p>
            </html>
        """.format("123456789 " * 38, "this-is-a-link-dont-cut-in-quotes")

        result = force_line_max_length(content, max_length_per_line=400, dont_cut_in_quotes=False)
        self.assertEqual(expected_content, result)



# -*- coding: utf-8 -*-
"""a simple store"""

from __future__ import unicode_literals

from datetime import datetime, time
from decimal import Decimal

from django.core.urlresolvers import reverse_lazy

from balafon.generic import StaffPopupFormView, StaffTemplateView
from balafon.Crm.models import Action
from balafon.Store.forms import AddExtraSaleForm
from balafon.Store.models import Sale, SaleItem


class StaticticsIndexView(StaffTemplateView):
    template_name = "Store/statistics_index.html"

    def get_context_data(self, **kwargs):
        context_data = super(StaticticsIndexView, self).get_context_data(**kwargs)

        try:
            first_sale = Sale.objects.all().order_by('action__planned_date')[0]
            from_year = first_sale.action.planned_date.year
        except IndexError:
            from_year = datetime.now().year

        context_data['from_year'] = from_year

        return context_data


class AddExtraSaleView(StaffPopupFormView):
    """This view makes possible to add other sales data than the one from the Internet site"""
    template_name = "Store/add_extra_sale.html"
    form_class = AddExtraSaleForm
    success_url = reverse_lazy('store_statistics_index')

    def form_valid(self, form):
        """create a new sale"""
        analysis_code = form.cleaned_data['analysis_code']
        planned_date = datetime.combine(form.cleaned_data['date'], time.min)
        amount = form.cleaned_data['amount']
        vat_rate = form.cleaned_data['vat_rate']

        action = Action.objects.create(type=analysis_code.action_type, planned_date=planned_date)

        if action.sale:
            action.sale.analysis_code = analysis_code
            action.sale.save()
            SaleItem.objects.create(
                sale=action.sale, pre_tax_price=amount, text=analysis_code.name, vat_rate=vat_rate, quantity=Decimal(1)
            )

        return super(AddExtraSaleView, self).form_valid(form)



# -*- coding: utf-8 -*-
"""REST api powered by django-rest-framework"""

from __future__ import unicode_literals

from rest_framework.response import Response
from rest_framework.views import APIView

from balafon.Store.models import Sale
from balafon.Store.api import get_staff_api_permissions, serializers


class SaleVatTotalsView(APIView):
    """get sales by category"""
    permission_classes = get_staff_api_permissions()

    def get_sale(self, sale_id):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return Sale.objects.get(id=sale_id)

    def get(self, request, **kwargs):
        """return a list with monthly amount by categories"""

        sale = self.get_sale(kwargs['sale_id'])

        vat_total_serializer = serializers.VatTotalSerializer(sale.vat_total_amounts(), many=True)
        return Response({'vat_totals': vat_total_serializer.data})

